#!/usr/bin/python2.7
# -*- coding: utf-8 -*-

from __future__ import print_function

import argparse
import os 
import sys
from oc_cdtapi.NexusAPI import *

def get_args():
    """
    Function for command line arguments parsing
    """
    parser = argparse.ArgumentParser(description="Simple Nexus API CLI")
    parser.add_argument("-url", "--mvn_url", type=str, help="Specify Nexus' URL (mandatory if MVN_URL environment variable is not set)")
    parser.add_argument("-urepo", "--upload_repo", type=str, help="Specify the upload repo")
    parser.add_argument("-drepo", "--download_repo", type=str, help="Specify the download repo")
    parser.add_argument("-user", "--mvn_user", type=str, help="Specify the Nexus username")
    parser.add_argument("-pass", "--mvn_password", type=str, help="Specify the Nexus user's password")
    parser.add_argument("gav", nargs="*", help="Specify one or multiple GAVs")
    parser.add_argument("-p", "--path", type=str, help="Specify the download path (default = current directory) or set the file path for upload")
    parser.add_argument("-f", "--filename", type=str, nargs="*", help="Specify the output filename(s) for download artifact(s)")
    parser.add_argument("-info", "--info", action = "store_true", help="Set the flag if you want to get info for the given GAV(s)")
    parser.add_argument("-d", "--download", action = "store_true", help="Set the flag if you want to instantly download given artifacts")
    parser.add_argument("-u", "--upload", action = "store_true", help="Set the flag if you want to instantly upload given artifact")
    parser.add_argument("-del", "--delete",action = "store_true", help="Set the flag if you want to delete given artifacts")
    parser.add_argument("--fail-on-artifact-exists", dest = 'no_check_exist', default = False, action = 'store_true', help = "Fail if artifact allready exists (otherwise - silently ignore it)")
    args = parser.parse_args()
    
    return parser,args

class ArtifactTool():
    """
    Simple command-line interface for working with Nexus API
    """

    def __init__(self, mvn_url=None, mvn_user=None, mvn_pass=None, testing=False):
        if testing:
            return
        
        self.API = NexusAPI(root=mvn_url, user=mvn_user, auth=mvn_pass)

    def get_info(self, gavs, download_repo=None):
        """
        Show information about the given GAV(s)
        """
        if not gavs:
            raise ValueError("No GAV was specified")

        for i in gavs:
            # and information itself in the console
            print(u"=======")
            print(u"GAV:\t'%s'" % i)
            print(u"Repo:\t'%s'" % (download_repo if download_repo else self.API.repo_default))
            print(u"URL:\t'%s'" % self.API.gav_get_url(i, download_repo))
            _exists = self.API.exists(i, repo=download_repo)
            print(u"Exists:\t'%s'" % str(_exists))

            if not _exists:
                return

            _info = self.API.info(i, download_repo)
            _ls = self.API.ls(i, download_repo)

            if _info:
                print(u"MD5:\t'%s'" % _info.get("md5"))
                print(u"MIME:\t'%s'" % _info.get("mime"))

            if _ls:
                print(u"LIST:\n '%s'" % "\n ".join(_ls))

    def check_existence(self, gav, repo=None):
        """
        Checks that the given GAV exists in the repository
        """
        # exception will be raised on caller methods (or here if GAV format is incorrect)
        return self.API.exists(gav, repo)

    def download_artifact(self, gavs, path=None, download_repo=None, result_filenames=None):
        """
        Downloads all given artifacts to the specified path
        """
        if not path:
            path = os.path.realpath(".")
        
        if not os.path.isdir(path):
            raise ValueError("Incorrect output path was specified.")

        if not gavs:
            raise ValueError("No GAV was specified")

        if result_filenames and len(gavs) != len(result_filenames):
            raise ValueError("GAVs and filenames quantity does not match")

        for i in gavs:
            if not self.check_existence(i, download_repo):
                raise FileNotFoundError(i)

            if not result_filenames:
                filename = gav_to_filename(i)
            else:
                filename = result_filenames[gavs.index(i)]

            with open(os.path.realpath(os.path.expanduser(path) + os.path.sep + filename), "wb") as output_file:
                self.API.cat(i, binary=True, stream=True, repo=download_repo, write_to=output_file)
                
            print(u"Downloaded: '{}' ==> '{}'".format(os.path.basename(i), filename))

    def delete_artifact(self, gavs=None, upload_repo=None):
        """
        Deletes one or multiple artifacts using their GAVs
        """
        if not gavs:
            raise ValueError("No GAV was specified")

        for i in gavs:
            print(self.API.remove(gav=i, repo=upload_repo))

    def upload_artifact(self, gav=None, path=None, upload_repo=None, no_check = True):
        """
        Uploads a new artifact
        """
        if not gav:
            raise ValueError("No GAV was specified")

        gav = gav[0]

        if not no_check and (self.check_existence(gav) or self.check_existence(gav, upload_repo)):
            print(u"'%s' already exists, skipping upload" % gav)
            return

        if path: 
            path = os.path.expanduser(path)
        
        if (not path) or (not os.path.isfile(path)):
            raise ValueError("Incorrect file was specified for upload")

        with open(os.path.realpath(path), "rb") as artifact:
            resp = self.API.upload(gav, data=artifact, metadata=True, repo=upload_repo)

        if "<Response [201]>" in str(resp):
            print(u"Uploaded: '{}' ==> '{}'".format(os.path.basename(path), gav))
        else:
            print(u"Couldn't upload the file.")

def main():
    """
    Main function of the CLI
    """

    parser,args = get_args()

    conn = ArtifactTool(args.mvn_url, args.mvn_user, args.mvn_password)

    if args.download and args.upload:
        raise ValueError(u"It's impossible to download and upload artifacts simultaneously.")

    if args.delete and (args.download or args.upload):
        raise ValueError(u"It's impossible to delete and download/upload artifacts simultaneously.")

    if args.download:
        conn.download_artifact(args.gav, args.path, args.download_repo, args.filename)
    elif args.upload:
        conn.upload_artifact(args.gav, args.path, args.upload_repo, args.no_check_exist)
    elif args.delete:
        conn.delete_artifact(args.gav, args.upload_repo)
    elif args.info:
        conn.get_info(args.gav, args.download_repo)
    else:
        parser.print_help()

        
if __name__ == '__main__':
    main()



import unittest
from oc_cdtapi.DevPIAPI import DevPIAPI;
import os;
import posixpath;
import xml.etree.ElementTree as ET;
import random;

## fake URL
str_fake_devpi_url = posixpath.join( "http:" + posixpath.sep + posixpath.sep, "fake.devpi.url", "fakerepo" );

## fake packages list
dict_fake_packages = {
        "blob-the-dog" : [ "1.0.0.24", "1.1.0.333", "1.1.1.1111" ],
        "get-out-of-here": [ "1.4.6.33451254", "1.4.7.44731" ]
        }

class _Response( object ):
    """
    Fake HTTP response
    """

    text = None;
    status_code = None;

class _DevPIAPI( DevPIAPI ):
    def _random_pkg_file( r_self, str_pkg, str_vers ):
        """
        generate random package file name
        """
        if ( random.randint( 0, 1 ) == 0 ):
            return "-".join( [ str_pkg, str_vers, "py2", "none", "any.whl" ] );

        return "-".join( [ str_pkg, str_vers + ".tar.gz" ] );


    def get( r_self, req, params = None, files = None, data = None, headers = None, **kvarg ):
        """
        Here we shall construct a fake HTML response
        """

        req = req.replace( str_fake_devpi_url, "" ).replace( "+simple", "" ).strip().strip( posixpath.sep );
        # here we should have a component only

        resp = _Response();
        resp.url = posixpath.join( str_fake_devpi_url, req );

        if req not in dict_fake_packages:
            resp.status_code = 404; # NOT FOUND
            resp.text = None;
            return resp;

        resp.status_code = 200; #OK

        xml_resp_root = ET.Element( "html" );
        xml_resp_body = ET.Element( "body" );
        xml_resp_root.append( xml_resp_body );

        for str_vers in dict_fake_packages[ req ]:
            xml_a_tag = ET.Element( "a" );
            str_full_pkg_file = r_self._random_pkg_file ( req, str_vers );
            xml_a_tag.attrib[ 'href' ] = posixpath.join( "..", "..", "+f", "00", "00", str_full_pkg_file );
            xml_a_tag.text = str_full_pkg_file;
            xml_resp_body.append( xml_a_tag );

        resp.text = ET.tostring( xml_resp_root );

        return resp;

class TestDevPIAPI( unittest.TestCase ):
    def test_get_versions( r_self ):
        os.environ[ "PYPI_PRODUCTION_URL" ] = str_fake_devpi_url;
        obj_api = _DevPIAPI();

        for str_pkg in dict_fake_packages.keys():
            test_result = True
            eth_arr = dict_fake_packages [str_pkg]
            cmp_arr = obj_api.get_package_versions (str_pkg)
            for eth_val in eth_arr:
                if not eth_val in cmp_arr:
                    test_result = False
            r_self.assertTrue (test_result)
#            r_self.assertEqual( dict_fake_packages[ str_pkg ], obj_api.get_package_versions( str_pkg ) );

        r_self.assertIsNone( obj_api.get_package_versions( "this_package-does_not-exist" ) );



import logging
import os
import re

from oc_cdtapi import API

class DmsAPIError(API.HttpAPIError):
    pass

class DmsAPI(API.HttpAPI):   # we use HttpAPI as a base class - the idea of HttpAPI is to use it as a skelet for new API clients
    """
    DmsAPI implementation
    """
    # do not forget about docstrings

    _env_prefix = 'DMS' # this automatically allows usage of DMS_* environment variables - everything is done in HttpAPI for you
    _env_token = '_TOKEN'
    _env_crs = '_CRS_URL' # now we have a separate Components Registry Service for components info obtaining

    def __init__(self, *args, **argv):
        """
        Initialiazing the parent class then loading the DMS API's bearer token
        """
        API.HttpAPI.__init__(self)
        self.crs_root = os.getenv(self._env_prefix + self._env_crs)
        token = os.getenv(self._env_prefix + self._env_token)
        if not self.crs_root:
            raise DmsAPIError("DMS API initialization failed. The components request url is not set")
        if token:
            self.headers = {"Authorization": "Bearer {}".format(token)}
        else:
            self.headers = {} # Empty headers dict is added for backwards-compatibility with bearer token functional

        if self.crs_root[-1] != '/': self.crs_root += '/'

    def re(self, req):
        """
        Re-defines default request formater, not to be called directly
        This forms request URL separated by / from string array
        """
        if len(req) == 0: return self.root
        return self.root + 'dms-service/rest/api/' + '/'.join(req)

        # directly accessing class variables outside the class itself is possible, but should be avoided
        # consider using setters/getters or storing state outside the class
    
    def crs_re(self, req):
        """
        Forming the correct URL for the DMS Components Registry Service
        """
        if len(req) == 0: return self.crs_root
        return self.crs_root + 'rest/api/' + '/'.join(req)

    def get_artifacts(self, component, version, ctype = None):
        """
        Gets list of artifacts of given component, version and type
        :param component: dms component name
        :param version:   dms version name
        :param ctype:     type of artifact. if not specified - query all known types
        :returns:         list of artifacts
        """
        assert bool(re.match('^[a-zA-Z0-9_-]*$', component)), "Component name must contain only latin letters, numbers, underscores and hyphens"
        assert bool(re.match('^[a-zA-Z0-9._-]*$', version)), "Version must contain only latin letters, numbers, underscores, hyphens and dots"
        logging.debug('Reached %s.get_artifacts', self.__class__.__name__)
        if ctype is None:
            types = self.get_types()
        else:
            assert bool(re.match('^[a-zA-Z0-9_-]*$', ctype)), "Component type must contain only latin letters, numbers, underscores and hyphens"
            types = [ ctype ]

        artifacts = []

        for t in types:
            req = ['2', 'component', component, 'version', version, t, 'list']
            artifacts += self.get(req, headers=self.headers, verify=False).json() # Why do you use write_to parameter instead of just using response object ?
            # also, requests has json parser, no need to re-invent it

        logging.debug('About to return an array of %d elements', len(artifacts)) # logging has its own format-string engine
        return artifacts


    def get_components(self):
        """
        Gets list of components known to DMS
        :returns: array of components
        """
        logging.debug('Reached %s.get_components', self.__class__.__name__)
        req = ['1', 'components']

        crs_request_url = self.crs_re(req)
        components = self.web.get(crs_request_url, verify=False).json()['components']
        logging.debug('About to return an array of %d elements', len(components))
        return components

    def get_gav(self, component, version, ctype, artifact, classifier=None):
        """
        Requests and forms gav for specified artifact
        :param component: component name
        :param version: version
        :param ctype:
        :param artifact:
        :param classifier:
        :returns: gav
        """
        assert bool(re.match('^[a-zA-Z0-9_-]+$', component)), "Component name have not to be empty and must contain only latin letters, numbers, underscores and hyphens"
        assert bool(re.match('^[a-zA-Z0-9\._-]+$', version)), "Version have not to be empty and must contain only latin letters, numbers, underscores, hyphens and dots"
        assert bool(re.match('^[a-zA-Z0-9_-]+$', ctype)), "Component type have not to be empty and must contain only latin letters, numbers, underscores and hyphens"
        assert bool(re.match('^[a-zA-Z0-9_-]+$', artifact)), "Artifact type have not to be empty and must contain only latin letters, numbers, underscores and hyphens"
        logging.debug('Reached %s.get_gav', self.__class__.__name__)
        logging.debug('component: {0}'.format(component))
        logging.debug('version: {0}'.format(version))
        logging.debug('artifact: {0}'.format(artifact))
        logging.debug('classifier: {0}'.format(classifier))
        req = ['1', 'component', component, 'version', version, ctype, artifact, 'gav']
        if classifier:
            assert bool(re.match('^[a-zA-Z0-9_-]+$', classifier)), "Non-empty classifier must contain only latin letters, hyphens and underscores"
            params = {'classifier': classifier}
        else:
            params = None

        gav = self.get(req, params, headers=self.headers).json()

        assert bool(re.match('^[a-zA-Z0-9\._-]+$', gav['groupId'])), "groupId have not to be empty and must contain only latin letters, numbers, underscores, hyphens and dots"
        assert bool(re.match('^[a-zA-Z0-9_-]+$', gav['artifactId'])), "artifactId have not to be empty and must contain only latin letters, numbers, underscores and hyphens"
        assert bool(re.match('^[a-zA-Z0-9\._-]+$', gav['version'])), "version have not to be empty and must contain only latin letters, numbers, underscores, hyphens and dots"
        assert bool(re.match('^[a-zA-Z0-9_-]+$', gav['packaging'])), "packaging have not to be empty and must contain only latin letters, hyphens and underscores"

        _gav = ':'.join( [gav['groupId'], gav['artifactId'], gav['version'], gav['packaging']] )
        if gav.get('classifier'):
            assert bool(re.match('^[a-zA-Z0-9_-]+$', gav['classifier'])), "non-empty classifier must contain only latin letters, hyphens and underscores"
            _gav = ':'.join([_gav, gav['classifier']])

        logging.debug('Formed gav: %s', _gav)

        return _gav


    def get_types(self): return ['notes', 'distribution', 'report', 'static', 'documentation']


    def get_versions(self, component):
        """
        fetches list of versions for specified component
        :param component: component name
        :returns: array of versions
        """
        assert bool(re.match('^[a-zA-Z0-9_-]*$', component)), "Component name must contain only latin letters, numbers, underscores and hyphens"
        logging.debug('Reached %s.get_versions', self.__class__.__name__)

        req = ['2', 'component', component, 'versions']

        versions = list(map(lambda x: x.get('version'), self.get(req, headers=self.headers).json().get('versions')))
        logging.debug('About to return an array of %d elements',len(versions))
        return versions


    def ping_dms(self):
        """
        sends a request to dms root
        :returns: server response
        """
        logging.debug('Reached %s.ping_dms', self.__class__.__name__)
        return self.get([], headers=self.headers).content




from unittest import TestCase
from oc_cdtapi import NexusAPI
from sys import version_info

if version_info.major == 2:
    import mock
else:
    from unittest import mock

import posixpath
import os
import tempfile
import re
import json

from .mocks.NexusAPILsMock import ArtifactoryAPILsMock

import logging
logging.getLogger().propagate = False
logging.getLogger().disabled = True

_af_url = "http://127.0.0.1:8082/artifactory"

class ArtifactoryAPITestSuite(TestCase):

    @mock.patch.dict("os.environ", {"MVN_URL": _af_url})
    @mock.patch('requests.Session')
    def setUp(self, session_mock):
        self.assertEqual(os.getenv("MVN_URL"), _af_url)
        self.api = NexusAPI.NexusAPI()
        self.assertIsInstance(self.api.web, mock.MagicMock)
        self.assertTrue(self.api.is_artifactory)
        self.assertFalse(self.api.is_nexus)
        self.put_headers = {'Content-Type': 'application/binary'}

    def test_init_bare(self):
        self.assertEqual(self.api.root.strip(posixpath.sep), _af_url.strip(posixpath.sep))

    @mock.patch.dict("os.environ", {"MVN_URL": _af_url})
    @mock.patch('requests.Session')
    def test_init_parms(self, session):
        _api = NexusAPI.NexusAPI(root=_af_url, 
                user="user",
                auth="password",
                upload_repo="upload",
                download_repo="download")

        self.assertEqual(_api.root.strip(posixpath.sep), _af_url.strip(posixpath.sep))
        self.assertEqual(_api.web.auth[0], "user")
        self.assertEqual(_api.web.auth[1], "password")

        # since __download_repo and __upload_repo are hidden from us
        _expected_put = posixpath.join(_af_url, "upload", NexusAPI.gav_to_path("g:a:v"))
        _retval_put = mock.MagicMock()
        _retval_put.status_code = 200
        _api.web.put = mock.MagicMock(return_value=_retval_put)
        _api.upload('g:a:v', data='test_data')

        _expected_get = posixpath.join(_af_url, "download", NexusAPI.gav_to_path("g:a:v"))
        _retval_get = mock.MagicMock()
        _retval_get.status_code = 200
        _retval_get.data = b"test_data"
        _api.web.get = mock.MagicMock(return_value=_retval_get)
        _api.cat('g:a:v')

        # urllib.urlparse.urljoin is not suitable for us since it joins only host part + relative part
        _api.web.put.assert_called_once_with(_expected_put, data='test_data', files=None, headers=self.put_headers, params=None)
        _api.web.get.assert_called_once_with(_expected_get, data=None, files=None, headers=None, params=None, stream=False)

    def test_cat_general(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        _ret.content = b"test_response"
        self.api.web.get = mock.MagicMock(return_value=_ret)
        _res = self.api.cat("g:a:v:p:c")
        _expected_get = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:v:p:c"))
        self.api.web.get.assert_called_once_with(_expected_get, data=None, files=None, headers=None, params=None, stream=False)
        self.assertEqual(_res, "test_response")

    @mock.patch.dict("os.environ", {"MVN_URL": _af_url, "MVN_DOWNLOAD_REPO": "download"})
    def test_cat_env(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        _ret.content = b"test_response"
        self.api.web.get = mock.MagicMock(return_value=_ret)
        _res = self.api.cat("g:a:v:p:c")
        _expected_get = posixpath.join(_af_url, "download", NexusAPI.gav_to_path("g:a:v:p:c"))
        self.api.web.get.assert_called_once_with(_expected_get, data=None, files=None, headers=None, params=None, stream=False)
        self.assertEqual(_res, "test_response")

    def test_cat_repo(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        _ret.content = b"test_response"
        self.api.web.get = mock.MagicMock(return_value=_ret)
        _res = self.api.cat("g:a:v:p:c", repo="download")
        _expected_get = posixpath.join(_af_url, "download", NexusAPI.gav_to_path("g:a:v:p:c"))
        self.api.web.get.assert_called_once_with(_expected_get, data=None, files=None, headers=None, params=None, stream=False)
        self.assertEqual(_res, "test_response")

    def test_cat_codepage(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        _ret.content = b"test_response_rus_\xf2\xe5\xf1\xf2\xee\xe2\xe0\xff_\xf5\xf0\xe5\xed\xfc"
        self.api.web.get = mock.MagicMock(return_value=_ret)
        _res = self.api.cat("g:a:v:p:c", encoding="cp1251")
        _expected_get = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:v:p:c"))
        self.api.web.get.assert_called_once_with(_expected_get, data=None, files=None, headers=None, params=None, stream=False)
        self.assertEqual(_res, b"test_response_rus_\xf2\xe5\xf1\xf2\xee\xe2\xe0\xff_\xf5\xf0\xe5\xed\xfc".decode("cp1251"))

    def test_cat_binary(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        _ret.content = b"test_response_bin"
        self.api.web.get = mock.MagicMock(return_value=_ret)
        _res = self.api.cat("g:a:v:p:c", binary=True)
        _expected_get = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:v:p:c"))
        self.api.web.get.assert_called_once_with(_expected_get, data=None, files=None, headers=None, params=None, stream=False)
        self.assertEqual(_res, b"test_response_bin")

    def test_cat_response(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        _ret.content = b"test_response_bin"
        self.api.web.get = mock.MagicMock(return_value=_ret)
        _res = self.api.cat("g:a:v:p:c", response=True)
        _expected_get = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:v:p:c"))
        self.api.web.get.assert_called_once_with(_expected_get, data=None, files=None, headers=None, params=None, stream=False)
        self.assertEqual(_res, _ret)

    # REST calls are to be ignored for Artifactory, so GET is expected to be called with
    # the same request as without 'rest_call' flag
    def test_cat_rest_call(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        _ret.content = b"test_response"
        self.api.web.get = mock.MagicMock(return_value=_ret)
        _res = self.api.cat("g:a:latest:p:c", rest_call=True)
        _expected_get = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:latest:p:c"))
        _expected_params = None
        self.api.web.get.assert_called_once_with(_expected_get, data=None, files=None, headers=None, params=_expected_params, stream=False)
        self.assertEqual(_res, "test_response")

    def test_cat_write_to(self):
        _flt = tempfile.NamedTemporaryFile()
        _ret = mock.MagicMock()
        _ret.status_code = 200
        _ret.content = b"test_response_write"
        self.api.web.get = mock.MagicMock(return_value=_ret)
        _res = self.api.cat("g:a:v:p:c", write_to=_flt.name)
        _expected_get = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:v:p:c"))
        self.api.web.get.assert_called_once_with(_expected_get, data=None, files=None, headers=None, params=None, stream=False)
        self.assertEqual(_res, _ret)
        _flt.seek(0, 0)
        self.assertEqual(b"test_response_write", _flt.read())

    def test_cat_rest_write_to(self):
        _flt = tempfile.NamedTemporaryFile()
        _ret = mock.MagicMock()
        _ret.status_code = 200
        _ret.content = b"test_response_write_rest_bin"
        self.api.web.get = mock.MagicMock(return_value=_ret)
        _res = self.api.cat("g:a:latest:p:c", rest_call=True, write_to=_flt.name)
        _expected_get = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:latest:p:c"))
        _expected_params = None
        self.api.web.get.assert_called_once_with(_expected_get, data=None, files=None, headers=None, params=_expected_params, stream=False)
        self.assertEqual(_res, _ret)
        _flt.seek(0, 0)
        self.assertEqual(b"test_response_write_rest_bin", _flt.read())

    def test_cat_encoding_error(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        _ret.content = b"test_response_rus_\xf2\xe5\xf1\xf2\xee\xe2\xe0\xff_\xf5\xf0\xe5\xed\xfc"
        # this is cp1251 legal string, but incorrect for utf-8
        self.api.web.get = mock.MagicMock(return_value=_ret)

        _res = None
        with self.assertRaises(UnicodeDecodeError):
            _res = self.api.cat("g:a:v:p:c", encoding="utf-8", enc_errors='strict')

        _expected_get = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:v:p:c"))
        self.api.web.get.assert_called_once_with(_expected_get, data=None, files=None, headers=None, params=None, stream=False)
        self.assertIsNone(_res)

    def test_cat_exception(self):
        _ret = mock.MagicMock()
        _ret.status_code = 404
        self.api.web.get = mock.MagicMock(return_value=_ret)

        with self.assertRaises(NexusAPI.NexusAPIError):
            _res = self.api.cat("g:a:v:p:c")

        _expected_get = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:v:p:c"))
        self.api.web.get.assert_called_once_with(_expected_get, data=None, files=None, headers=None, params=None, stream=False)

    def test_exists_general(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        self.api.web.head = mock.MagicMock(return_value=_ret)
        _expected_head = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:v:p:c"))
        self.assertTrue(self.api.exists("g:a:v:p:c"))
        self.api.web.head.assert_called_once_with(_expected_head, params=None, files=None, data=None, headers=None)
        self.api.web.get.assert_not_called()

    def test_exists_repo(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        self.api.web.head = mock.MagicMock(return_value=_ret)
        _repo = 'some_repo'
        _expected_head = posixpath.join(_af_url, _repo, NexusAPI.gav_to_path("g:a:v:p:c"))
        self.assertTrue(self.api.exists("g:a:v:p:c", repo=_repo))
        self.api.web.head.assert_called_once_with(_expected_head, params=None, files=None, data=None, headers=None)
        self.api.web.get.assert_not_called()

    @mock.patch.dict("os.environ", {"MVN_URL": _af_url, "MVN_DOWNLOAD_REPO": "download"})
    def test_exists_env(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        self.api.web.head = mock.MagicMock(return_value=_ret)
        _repo = 'download'
        _expected_head = posixpath.join(_af_url, _repo, NexusAPI.gav_to_path("g:a:v:p:c"))
        self.assertTrue(self.api.exists("g:a:v:p:c"))
        self.api.web.head.assert_called_once_with(_expected_head, params=None, files=None, data=None, headers=None)
        self.api.web.get.assert_not_called()

    # REST calls are to be ignored for Artifactory
    def test_exists_rest(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        self.api.web.head = mock.MagicMock(return_value=_ret)
        _expected_head = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:v:p:c"))
        self.assertTrue(self.api.exists("g:a:v:p:c", rest_call=True))
        self.api.web.head.assert_called_once_with(_expected_head, params=None, files=None, data=None, headers=None)
        self.api.web.get.assert_not_called()

    def test_not_exists_general(self):
        _ret = mock.MagicMock()
        _ret.status_code = 404
        self.api.web.head = mock.MagicMock(return_value=_ret)
        _expected_head = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:v:p:c"))
        self.assertFalse(self.api.exists("g:a:v:p:c"))
        self.api.web.head.assert_called_once_with(_expected_head, params=None, files=None, data=None, headers=None)
        self.api.web.get.assert_not_called()

    def test_not_exists_repo(self):
        _ret = mock.MagicMock()
        _ret.status_code = 404
        self.api.web.head = mock.MagicMock(return_value=_ret)
        _repo = 'some_repo'
        _expected_head = posixpath.join(_af_url, _repo, NexusAPI.gav_to_path("g:a:v:p:c"))
        self.assertFalse(self.api.exists("g:a:v:p:c", repo=_repo))
        self.api.web.head.assert_called_once_with(_expected_head, params=None, files=None, data=None, headers=None)
        self.api.web.get.assert_not_called()

    @mock.patch.dict("os.environ", {"MVN_URL": _af_url, "MVN_DOWNLOAD_REPO": "download"})
    def test_not_exists_env(self):
        _ret = mock.MagicMock()
        _ret.status_code = 404
        self.api.web.head = mock.MagicMock(return_value=_ret)
        _repo = 'download'
        _expected_head = posixpath.join(_af_url, _repo, NexusAPI.gav_to_path("g:a:v:p:c"))
        self.assertFalse(self.api.exists("g:a:v:p:c"))
        self.api.web.head.assert_called_once_with(_expected_head, params=None, files=None, data=None, headers=None)
        self.api.web.get.assert_not_called()

    # should be the same as without 'rest_call' flag
    def test_not_exists_rest(self):
        _ret = mock.MagicMock()
        _ret.status_code = 404
        self.api.web.head = mock.MagicMock(return_value=_ret)
        _expected_head = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:v:p:c"))
        self.assertFalse(self.api.exists("g:a:v:p:c", rest_call=True))
        self.api.web.get.assert_not_called()
        self.api.web.head.assert_called_once_with(_expected_head, params=None, files=None, data=None, headers=None)

    def test_exists_error(self):
        _ret = mock.MagicMock()
        _ret.status_code = 418
        self.api.web.head = mock.MagicMock(return_value=_ret)
        _expected_head = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:v:p:c"))
        with self.assertRaises(NexusAPI.NexusAPIError):
            (self.api.exists("g:a:v:p:c"))

        self.api.web.head.assert_called_once_with(_expected_head, params=None, files=None, data=None, headers=None)
        self.api.web.get.assert_not_called()

    # should be the same as without rest for Artifactory
    def test_exists_rest_error(self):
        _ret = mock.MagicMock()
        _ret.status_code = 418
        self.api.web.head = mock.MagicMock(return_value=_ret)
        _expected_head = posixpath.join(_af_url, "maven-virtual", NexusAPI.gav_to_path("g:a:v:p:c"))

        with self.assertRaises(NexusAPI.NexusAPIError):
            (self.api.exists("g:a:v:p:c", rest_call=True))

        self.api.web.get.assert_not_called()
        self.api.web.head.assert_called_once_with(_expected_head, params=None, files=None, data=None, headers=None)

    def test_upload_general(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        self.api.web.put = mock.MagicMock(return_value=_ret)
        self.api.update_metadata = mock.MagicMock()
        res = self.api.upload('a.b.c.d:artifact:1.1', repo='repoid', data='HELLO, WORLD!!!', pom = None)
        self.assertEqual(res, _ret)
        _expected_put = posixpath.join(_af_url, 'repoid', 'a', 'b', 'c', 'd', 'artifact', '1.1', 'artifact-1.1.jar')
        self.api.web.put.assert_called_once_with(_expected_put, data='HELLO, WORLD!!!', files=None, headers=self.put_headers, params=None)
        self.api.update_metadata.assert_not_called()

    def test_upload_repo(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        self.api.web.put = mock.MagicMock(return_value=_ret)
        self.api.update_metadata = mock.MagicMock()
        res = self.api.upload('a.b.c.d:artifact:1.1', repo='repositories/id', data='HELLO, WORLD!!!', pom = None)
        self.assertEqual(res, _ret)
        _expected_put = posixpath.join(_af_url, 'id', 'a', 'b', 'c', 'd', 'artifact', '1.1', 'artifact-1.1.jar')
        self.api.web.put.assert_called_once_with(_expected_put, data='HELLO, WORLD!!!', files=None, headers=self.put_headers, params=None)
        self.api.update_metadata.assert_not_called()

    def test_upload_pom(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        self.api.web.put = mock.MagicMock(return_value=_ret)
        self.api.update_metadata = mock.MagicMock()
        res = self.api.upload('a.b.c.d:artifact:1.1','repositories/id','HELLO, WORLD!!!', pom = '<pom>this is a pom</pom>')
        _expected_put = posixpath.join(_af_url, 'id', 'a', 'b', 'c', 'd', 'artifact', '1.1', 'artifact-1.1')
        self.assertEqual(self.api.web.put.call_count, 2)
        self.api.web.put.assert_any_call(_expected_put + '.jar', data='HELLO, WORLD!!!', files=None, headers=self.put_headers, params=None)
        self.api.web.put.assert_any_call(_expected_put + '.pom', data='<pom>this is a pom</pom>', files=None, headers=self.put_headers, params=None)
        self.assertEqual(res, _ret)
        self.api.update_metadata.assert_not_called()

    def test_upload_pom_generate(self):
        _gav = 'a.b.c.d:artifact:1.1:jar:classifier'

        # We do not need to test pom generation here. There are separate tests for it.
        pom_data_expected = NexusAPI.pom_from_gav(_gav)
        _ret = mock.MagicMock()
        _ret.status_code = 200
        self.api.web.put = mock.MagicMock(return_value=_ret)
        self.api.exists = mock.MagicMock(return_value=False)
        self.api.update_metadata = mock.MagicMock()

        res = self.api.upload(_gav, repo='id', data=b'HELLO, WORLD!!!', pom=True)
        self.assertEqual(res, _ret)
        _expected_exists = NexusAPI.parse_gav(_gav)
        _expected_exists['p'] = 'pom'
        _expected_put = posixpath.join(_af_url, 'id', 'a', 'b', 'c', 'd', 'artifact', '1.1', 'artifact-1.1-classifier')
        self.api.exists.assert_called_once_with(_expected_exists, repo='id')
        self.assertEqual(2, self.api.web.put.call_count)
        self.api.web.put.assert_any_call(_expected_put + '.jar', data=b'HELLO, WORLD!!!', files=None, headers=self.put_headers, params=None)
        self.api.web.put.assert_any_call(_expected_put + '.pom', data=pom_data_expected, files=None, headers=self.put_headers, params=None)
        self.api.update_metadata.assert_not_called()

    # metadata updating is not supported for Artifactory
    def test_upload_pom_generate_with_metadata(self):
        _gav = 'a.b.c.d:artifact:1.1:zip:classifier'
        pom_data_expected = NexusAPI.pom_from_gav(_gav)
        _ret = mock.MagicMock()
        _ret.status_code = 200
        self.api.web.put = mock.MagicMock(return_value=_ret)
        self.api.exists = mock.MagicMock(return_value=False)
        self.api.update_metadata = mock.MagicMock()

        res = self.api.upload(_gav, repo='id', data=b'HELLO, WORLD!!!', pom=True, metadata=True)
        self.assertEqual(res, _ret)
        _expected_exists = NexusAPI.parse_gav(_gav)
        _expected_exists['p'] = 'pom'
        _expected_put = posixpath.join(_af_url, 'id', 'a', 'b', 'c', 'd', 'artifact', '1.1', 'artifact-1.1-classifier')
        self.api.exists.assert_called_once_with(_expected_exists, repo='id')
        self.assertEqual(2, self.api.web.put.call_count)
        self.api.web.put.assert_any_call(_expected_put + '.zip', data=b'HELLO, WORLD!!!', files=None, headers=self.put_headers, params=None)
        self.api.web.put.assert_any_call(_expected_put + '.pom', data=pom_data_expected, files=None, headers=self.put_headers, params=None)
        self.api.update_metadata.assert_not_called()

    def test_upload_pom_exists(self):
        _gav = 'a.b.c.d:artifact:1.1:zip:classifier'
        _ret = mock.MagicMock()
        _ret.status_code = 200
        self.api.web.put = mock.MagicMock(return_value=_ret)
        self.api.exists = mock.MagicMock(return_value=True)
        self.api.update_metadata = mock.MagicMock()

        res = self.api.upload(_gav, repo='id', data=b'HELLO, WORLD!!!', pom=True)
        self.assertEqual(res, _ret)
        _expected_exists = NexusAPI.parse_gav(_gav)
        _expected_exists['p'] = 'pom'
        _expected_put = posixpath.join(_af_url, 'id', 'a', 'b', 'c', 'd', 'artifact', '1.1', 'artifact-1.1-classifier.zip')
        self.api.exists.assert_called_once_with(_expected_exists, repo='id')
        self.api.web.put.assert_called_once_with(_expected_put, data=b'HELLO, WORLD!!!', files=None, headers=self.put_headers, params=None)
        self.api.update_metadata.assert_not_called()

    # metadata updating is supported for Nexus only
    @mock.patch.dict("os.environ", {"MVN_URL": _af_url, "MVN_UPLOAD_REPO": "upload"})
    def test_updatemeta_general_env(self):
        self.assertIsNone(self.api.update_metadata())
        self.api.web.delete.assert_not_called()

    def test_updatemeta_repo(self):
        _gav = "a.b.c.d.e.f:aIdS:6.7.8.9.10:puck"
        self.assertIsNone(self.api.update_metadata(gav=_gav, repo='upload'))
        self.api.web.delete.assert_not_called()

    def test_delete_general(self):
        _ret = mock.MagicMock()
        _ret.status_code = 200
        self.api.web.delete = mock.MagicMock(return_value=_ret)
        _rsp = self.api.remove(repo="repo")

        _del_expected = posixpath.join(_af_url, "repo")
        self.api.web.delete.assert_called_once_with(_del_expected, data=None, files=None, headers=None, params=None)

    @mock.patch.dict("os.environ", {"MVN_URL": _af_url, "MVN_UPLOAD_REPO": "upload"})
    def test_delete_env(self):
        _gav = "g:a:v"
        _ret = mock.MagicMock()
        _ret.status_code = 200
        self.api.web.delete = mock.MagicMock(return_value=_ret)
        _rsp = self.api.remove(gav=_gav)

        _del_expected = posixpath.join(_af_url, "upload", NexusAPI.gav_to_path(_gav))
        self.api.web.delete.assert_called_once_with(_del_expected, data=None, files=None, headers=None, params=None)

    def test_delete_repo(self):
        _gav = "g:a:v:p"
        _ret = mock.MagicMock()
        _ret.status_code = 200
        self.api.web.delete = mock.MagicMock(return_value=_ret)
        _rsp = self.api.remove(gav=_gav, repo="idUpload")

        _del_expected = posixpath.join(_af_url, "idUpload", NexusAPI.gav_to_path(_gav))
        self.api.web.delete.assert_called_once_with(_del_expected, data=None, files=None, headers=None, params=None)

    def test_delete_exc(self):
        _gav = "g:a:v:p:c"
        _ret = mock.MagicMock()
        _ret.status_code = 403
        self.api.web.delete = mock.MagicMock(return_value=_ret)

        with self.assertRaises(NexusAPI.NexusAPIError):
            _rsp = self.api.remove(gav=_gav, repo="idXupload")

        _del_expected = posixpath.join(_af_url, "idXupload", NexusAPI.gav_to_path(_gav))
        self.api.web.delete.assert_called_once_with(_del_expected, data=None, files=None, headers=None, params=None)


    def test_info_existent(self):
        _resp_json = """
        {
            "mimeType": "mime-type-g",
            "checksums":
            {
                "md5" : "123456abcdef"
            }
        }
        """
        _gav = "g:a:v:p"
        _ret = mock.MagicMock()
        _ret.status_code = 200
        _ret.content = _resp_json
        _ret.json = mock.MagicMock(return_value=json.loads(_resp_json))
        self.api.web.get = mock.MagicMock(return_value=_ret)

        self.assertEqual(self.api.info(_gav), {"md5": "123456abcdef", "mime": "mime-type-g"})
        _expected_parms = None
        _expected_get = posixpath.join(_af_url, "api", "storage", "maven-virtual", NexusAPI.gav_to_path(_gav))
        self.api.web.get.assert_called_once_with(_expected_get, data=None, files=None, headers=None, params=_expected_parms)

    def test_info_nonexistent(self):
        _resp_json = ""
        _gav = "g:a:v:p"
        _ret = mock.MagicMock()
        _ret.status_code = 404
        _ret.content = _resp_json
        self.api.web.get = mock.MagicMock(return_value=_ret)

        self.assertIsNone(self.api.info(_gav))
        _expected_parms = None
        _expected_get = posixpath.join(_af_url, "api", "storage", "maven-virtual", NexusAPI.gav_to_path(_gav))
        self.api.web.get.assert_called_once_with(_expected_get, data=None, files=None, headers=None, params=_expected_parms)

    def test_ls_nothing(self):
        self.api.web = ArtifactoryAPILsMock(_af_url)
        self.assertEqual(0, len(self.api.ls("surely.not.existent:artifact:9.0.9:zip")))

    def test_ls_exact(self):
        self.api.web = ArtifactoryAPILsMock(_af_url)
        _gav = "test.group.0.id:artifact-0:0.0.0:zip:cl"
        _ls = self.api.ls(_gav) 
        self.assertEqual(1, len(_ls))
        self.assertEqual(_ls.pop(), _gav)

    def test_ls_group(self):
        self.api.web = ArtifactoryAPILsMock(_af_url)
        _gav = "test.group.1.id"
        _ls = self.api.ls(_gav)
        _g_len = len(_ls)

        for _gv in _ls:
            self.assertTrue(_gv.startswith(_gav + ":"))

        _gav = ':'.join([_gav, "artifact-2"])
        _ls = self.api.ls(_gav)
        _a_len = len(_ls)
        self.assertTrue(_g_len >= _a_len)

        for _gv in _ls:
            self.assertTrue(_gv.startswith(_gav + ":"))

        _gav = ':'.join([_gav, "0.0.0"])
        _ls = self.api.ls(_gav)
        _v_len = len(_ls)
        self.assertTrue(_a_len >= _v_len)

        for _gv in _ls:
            self.assertTrue(_gv.startswith(_gav + ":"))

        _gav = ':'.join([_gav, "jar"])
        _ls = self.api.ls(_gav)
        _p_len = len(_ls)
        self.assertTrue(_v_len >= _p_len)

        for _gv in _ls:
            self.assertTrue(_gv.startswith(_gav))

        _gav = ':'.join([_gav, "cl"])
        self.assertEqual(_gav, self.api.ls(_gav).pop())

    def test_ls_filter(self):
        self.api.web = ArtifactoryAPILsMock(_af_url)
        _gav = 'test.group.2.id:::zip'
        _filter = 'artifact-[0-9]+:[^:]+:zip'
        _rfl = re.compile(_filter)

        _ls = self.api.ls(_gav, filter=_filter)

        for _art in _ls:
            _gg = NexusAPI.parse_gav(_art)
            self.assertEqual(_gg.get('g'), 'test.group.2.id')
            self.assertEqual(_gg.get('p'), 'zip')
            self.assertIsNotNone(_rfl.search(_art))
            
    def test_ls_filter_revert(self):
        self.api.web = ArtifactoryAPILsMock(_af_url)
        _gav = 'test.group.2.id::'
        _filter = 'artifact-[0-9]+:[^:]+:zip'
        _rfl = re.compile(_filter)

        _ls = self.api.ls(_gav, filter=_filter, filter_revert=True)

        for _art in _ls:
            _gg = NexusAPI.parse_gav(_art)
            self.assertEqual(_gg.get('g'), 'test.group.2.id')
            self.assertNotEqual(_gg.get('p'), 'zip')
            self.assertIsNone(_rfl.search(_art))

    def test_gav_url(self):
        self.assertEqual(self.api.gav_get_url('groupId:artifactId:version:packaging'), 
                posixpath.join(_af_url, "maven-virtual", "groupId",
                    "artifactId", "version", "artifactId-version.packaging"))

        self.assertEqual(self.api.gav_get_url('groupId:artifactId:version:packaging:classifiler'), 
                posixpath.join(_af_url, "maven-virtual", "groupId",
                    "artifactId", "version", "artifactId-version-classifiler.packaging"))

        self.assertEqual(self.api.gav_get_url('group.Id:artifactId:version:packaging:classifiler'), 
                posixpath.join(_af_url, "maven-virtual", "group", "Id",
                    "artifactId", "version", "artifactId-version-classifiler.packaging"))

        self.assertEqual(self.api.gav_get_url('group.Id:artifactId:version:packaging:classifiler', repo="hellp"), 
                posixpath.join(_af_url, "hellp", "group", "Id",
                    "artifactId", "version", "artifactId-version-classifiler.packaging"))

        with self.assertRaises(ValueError):
            self.api.gav_get_url(None, repo="hellp")

        with self.assertRaises(ValueError):
            self.api.gav_get_url(None)



import sys
import io
import tempfile
import os

from unittest import TestCase
from nexus import ArtifactTool
from oc_cdtapi.NexusAPI import parse_gav


class MockAPI():
    """
    Mocked NexusAPI class for testing purposes
    """

    def check_gav(self, gav):
        try:
            parse_gav(gav)
        except ValueError:
            return False
        else:
            return True

    def info(self, gav, repo=None):
        if not self.check_gav(gav):
            raise ValueError("Incorrect GAV was provided")

    def ls(self, gav, repo=None):
        pass

    def exists(self, gav, repo=None):
        ex = self.check_gav(gav)
        return ex

    def cat(self, *args, **kwargs):
        pass

    def delete(self, *args, **kwargs):
        res = "<Response [204]>"
        return res

    def update_metadata(self, *args, **kwargs):
        pass

    def upload(self, gav, data=None, metadata=None, repo=None):
        if not repo:
            raise ValueError("You must provide repo name")
        res = "<Response [201]>"
        return res

    @property
    def repo_default(self):
        return "default"

    def gav_get_url(self, gav, repo=None):
        return "default_url"

    def remove(self, gav, repo=None):
        if not repo:
            raise SystemExit(1)

        if sys.version_info.major >= 3:
            return "'%s' has been deleted" % gav

        return unicode("'%s' has been deleted" % gav)

class ArtifactToolTestSuite(TestCase):

    def setUp(self):
        self.conn = ArtifactTool(testing=True)
        self.conn.API = MockAPI()
        self.cout = io.StringIO()
        sys.stdout = self.cout
        self.tempfile = tempfile.NamedTemporaryFile(delete=True)
        os.environ["MVN_UPLOAD_REPO"] = ""

    def tearDown(self):
        self.tempfile.close()

    def test_no_gav_provided(self):
        with self.assertRaises(ValueError):
            self.conn.download_artifact(None)
            self.conn.delete_artifact(None)
            self.conn.upload_artifact(None)
            self.conn.get_info(None)

    def test_get_info_multiple_gavs(self):
        self.conn.get_info(["GG:AA:VV", "GG1:AA1:VV1"])
        if sys.version_info.major == 2:
            self.assertRegexpMatches(self.cout.getvalue(), "GG:AA:VV")
        else:
            self.assertRegex(self.cout.getvalue(), "GG:AA:VV")

    def test_check_existence_correct_gav(self):
        self.assertTrue(self.conn.check_existence("GG:AA:VV"))

    def test_download_artifact_no_path_provided(self):
        self.conn.download_artifact(["GG:AA:VV"])
        _check_msg = "Downloaded: 'GG:AA:VV'"
        if sys.version_info.major == 2:
            self.assertRegexpMatches(self.cout.getvalue(), _check_msg)
        else:
            self.assertRegex(self.cout.getvalue(), _check_msg)

    def test_download_artifact_not_dir_in_path(self):
        with self.assertRaises(ValueError):
            self.conn.download_artifact(["GG:AA:VV"], self.tempfile.name)

    def test_download_artifact_multiple_gavs(self):
        _list = ["GG:AA:VV", "GG1:AA1:VV1"]
        self.conn.download_artifact(_list, ".")

        if sys.version_info.major == 2:
            _check_mthd = self.assertRegexpMatches
        else:
            _check_mthd = self.assertRegex

        for _art in _list:
            _check_mthd(self.cout.getvalue(), "Downloaded: '%s'" % _art)

    def test_download_artifact_filename_provided(self):
        self.conn.download_artifact(["GG:AA:VV"], result_filenames=["test_file.zip"])
        if sys.version_info.major == 2:
            self.assertRegexpMatches(self.cout.getvalue(), "test_file.zip")
        else:
            self.assertRegex(self.cout.getvalue(), "test_file.zip")

    def test_download_artifact_gavs_filenames_quantity_mismatch(self):
        with self.assertRaises(ValueError):
            self.conn.download_artifact(["GG:AA:VV","GG1:AA1:VV1"], result_filenames=["test_file.zip"])

    def test_download_artifact_multiple_gavs_filenames(self):
        _arts = {"GG:AA:VV": "test_file1.zip", "GG1:AA1:VV1": "test_file2.zip"}
        self.conn.download_artifact(list(_arts.keys()), result_filenames=list(_arts.values()))

        if sys.version_info.major == 2:
            _check_mthd = self.assertRegexpMatches
        else:
            _check_mthd = self.assertRegex

        for _art, _file in _arts.items():
            _check_mthd(self.cout.getvalue(), "Downloaded: '%s' ==> '%s'" % (_art, _file))

    def test_delete_artifact_no_upload_repo(self):
        with self.assertRaises(SystemExit) as ec: 
            self.conn.delete_artifact(["GG:AA:VV", "GG1:AA1:VV1"])
        exit = ec.exception
        self.assertEqual(exit.code, 1)

    def test_delete_artifact_multiple_gavs(self):
        self.conn.delete_artifact(["GG:AA:VV", "GG1:AA1:VV1"], "test")
        if sys.version_info.major == 2:
            self.assertRegexpMatches(self.cout.getvalue(), "has been deleted")
        else:
            self.assertRegex(self.cout.getvalue(), "has been deleted")

    def test_upload_artifact_incorrect_file(self):
        with self.assertRaises(ValueError):
            self.conn.upload_artifact(["GG:AA:VV"], None, "test")

    def test_upload_artifact_no_upload_repo(self):
        with self.assertRaises(ValueError) as ec:
            self.conn.upload_artifact(["GG:AA:VV"], self.tempfile.name, None)

    def test_upload_artifact_multiple_gavs(self):
        _list = ["GG:AA:VV", "GG1:AA1:VV1"]
        self.conn.upload_artifact(_list, self.tempfile.name, "test")
        
        if sys.version_info.major == 2:
            _assert = self.assertRegexpMatches
        else:
            _assert = self.assertRegex

        _assert(self.cout.getvalue(), "^Uploaded: .* ==> '%s'$" % _list.pop(0))



from setuptools import setup, find_packages
import unittest
from doctest import DocFileSuite
import os
from sys import version_info
from datetime import datetime

def dynamic_version(str_version):
    """
    Returns full version of a package
    :param str_version: string, package version to append
    :return: str_version with appended build_id
    """

    str_vfile = 'version.txt'

    if not os.path.exists(str_vfile):
        fl_out = open(str_vfile, 'w')
        fl_out.write(datetime.strftime(datetime.now(), "%Y%m%d%H%M%S"))
        fl_out.close()

    fl_in = open(str_vfile)
    str_bid = fl_in.read().strip()
    fl_in.close()

    return '.'.join([str_version, str_bid])


MAJOR = 3
MINOR = 8 
RELEASE = 1

install_requires = ["requests", "mock"]
tests_require = []

if version_info.major == 2:
    install_requires.append("enum")
    tests_require.append("mock==2.0.0")

spec = {
    "name": "oc_cdtapi",
    "version": dynamic_version('.'.join(list(map(lambda x: str(x), [MAJOR, MINOR, RELEASE])))),
    "license": "Apache2.0",
    "description": "Custom Development python API libraries",
    "packages": find_packages(),
    "install_requires": install_requires,
    "tests_require": tests_require,
    "scripts": [
        "nexus.py"
    ],

}

setup(**spec)



import unittest
from oc_cdtapi.JenkinsAPI import Build, QueueItem, BuildStatus, BuildStatusException, JenkinsError, Jenkins
from requests import Response
from collections import namedtuple, OrderedDict
import re, posixpath;
import json;
import os;
import sys


MockResponse=namedtuple("MockResponse", ("status_code", "headers", "text", ))


class TestClient(object):

    def __init__(self, responses=[MockResponse(200, {}, ""), ]):
        self.responses=iter(responses)

    def get(self, relative_path, *args, **kwargs):
        return next(self.responses)

class TestJenkinsClient( Jenkins ):
    ls_jobs_disabled = list();
    dict_jobs_exist = dict();
    ## responses on slave-specific data
    computer_data = {
                'master': None,
                'test-node' : {"executors":[
                {"currentExecutable":{  "actions":[{},{},{},{},{},{}],
                                        "artifacts":[],
                                        "building":True,
                                        "description":None,
                                        "duration":0,
                                        "estimatedDuration":1706173,
                                        "executor":{},
                                        "fullDisplayName":"c.TEST_CUSTOMER.branches-test.test #27",
                                        "id":"2018-11-21_09-51-15",
                                        "keepLog":False,
                                        "number":27,
                                        "result":None,
                                        "timestamp":1542783075808,
                                        "url":"http://localhost:8080/job/c.TEST_CUSTOMER.branches-test.test/27/",
                                        "builtOn":"test-node",
                                        "changeSet":{},
                                        "culprits":[{}],
                                        "fingerprint":[]},
                "idle":False}]},
            'other-node': {"executors":[
                {"currentExecutable":{  "actions":[{},{},{},{},{},{}],
                                        "artifacts":[],
                                        "building":True,
                                        "description":None,
                                        "duration":0,
                                        "estimatedDuration":1706173,
                                        "executor":{},
                                        "fullDisplayName":"c.TEST_CUSTOMER.trunk.test #3",
                                        "id":"2018-11-21_09-51-15",
                                        "keepLog":False,
                                        "number":3,
                                        "result":None,
                                        "timestamp":154383075808,
                                        "url":"http://localhost:8080/job/c.TEST_CUSTOMER.trunk.test/3/",
                                        "builtOn":"other-node",
                                        "changeSet":{},
                                        "culprits":[{}],
                                        "fingerprint":[]},
                "idle":False}]}
            }

    queue_data = { "items":
                [   {"actions":[
                        {"causes":[{"shortDescription":"Started by an SCM change"},{"shortDescription":"Started by an SCM change"}]},
                        {"parameters":[{"name":"OWS_HOME","value":"/local/homes/ows_home"},{"name":"action","value":"reinstall"},{"name":"version","value":"default"}]}],
                    "blocked":True,
                    "buildable":False,
                    "id":9590,
                    "inQueueSince":1542791418408,
                    "params":"\nOWS_HOME=/local/homes/ows_home\naction=reinstall\nversion=default",
                    "stuck":False,
                    "task":{    "name":"c.OWI_LUXCTRL.branches-int.cards",
                                "url":"http://localhost:8080/job/c.OWI_LUXCTRL.branches-int.cards/",
                                "color":"red_anime" },
                    "url":"queue/item/9590/",
                    "why":"Build 9589 is in progress",
                    "buildableStartMilliseconds":1542791433530},
                    {"actions":[
                        {"parameters":[{"name":"REINSTALLORACLE","value":False},{"name":"DOCLEANUP","value":False}]},
                        {},
                        {"causes":[{"shortDescription":"Started by upstream project \"tech.selfcheck\" build number 38,163",
                            "upstreamBuild":38163,
                            "upstreamProject":"tech.selfcheck",
                            "upstreamUrl":"job/tech.selfcheck/"}]}],
                    "blocked":False,
                    "buildable":True,
                    "id":9559,
                    "inQueueSince":1542790144130,
                    "params":"\nREINSTALLORACLE=False\nDOCLEANUP=False",
                    "stuck":True,
                    "task":{    "name":"hosts=cislaveora-25",
                                "url":"http://localhost:8080/job/tech.selfcheck/hosts=cislaveora-25/","color":"blue"},
                    "url":"queue/item/9559/",
                    "why":"Waiting for next available executor on cislaveora-25",
                    "buildableStartMilliseconds":1542790144130,
                    "pending":False}] }

    def _computer_data_to_jobs_data( self ):
        dict_result = dict();
        dict_result[ "jobs" ] = list();
        
        for t in self.computer_data.keys():
            if self.computer_data[ t ] is None:
                continue;

            if "executors" not in self.computer_data[ t ]:
                continue;

            for executor in self.computer_data[ t ][ "executors" ]:

                if executor is None or "currentExecutable" not in executor:
                    continue;

                job_name = executor[ "currentExecutable" ][ "fullDisplayName" ].split( " " )[ 0 ];
                dict_job = { "name" : job_name, "lastBuild" : executor[ "currentExecutable" ] };
                dict_job[ "lastBuild" ][ "builtOn" ] = t;
                dict_result[ "jobs" ].append( dict_job );

        return dict_result;

    class GetResponse( object ):
        text = None;

    last_post_url = None;
    last_post_data = None;

    def post( self, str_url, **kwargs ):
        self.last_post_data = kwargs;
        self.last_post_url = str_url;
        return str_url, kwargs;

    def get( self, str_url, *args, **kwargs ):
        if str_url is None or len( str_url ) == 0:
            obj_resp = self.GetResponse();
            obj_resp.text = json.dumps( self._computer_data_to_jobs_data() );
            return obj_resp;

        if str_url.startswith( 'queue' ):
            str_rq = str_url.rstrip('queue').lstrip( posixpath.join( 'api','json' )).strip( posixpath.sep );

            if len( str_rq ) == 0:
                obj_resp = self.GetResponse();
                obj_resp.text = json.dumps( self.queue_data );
                return obj_resp;

            if str_rq.startswith( 'item' ):
                int_item_id = int( str_rq.rstrip( 'item' + posixpath.sep ).strip().strip( posixpath.sep ) );

                for dict_item in self.queue_data[ 'items' ]:
                    if not dict_item[ "id" ] == int_item_id:
                        continue;
                    
                    obj_resp = self.GetResponse();
                    obj_resp.text = json.dumps( dict_item );
                    return obj_resp;

                return self.GetResponse();

        if str_url.startswith( 'computer' ):
            if posixpath.sep not in str_url.strip().strip( posixpath.sep ):
                obj_resp = self.GetResponse();
                ls_resp = map ( lambda x: { "displayName" : x }, self.computer_data.keys() );
                str_json = { "computer" : list (ls_resp) }
                obj_resp.text = json.dumps( str_json );
#                obj_resp.text = json.dumps( { "computer" : ls_resp } );
                return obj_resp;

            ls_comps = str_url.split( posixpath.sep );
            str_comp_name = ls_comps[1];

            if len( ls_comps ) < 3:
                obj_resp = self.GetResponse();
                obj_resp.text = json.dumps( self.computer_data[ str_comp_name ] );
                return obj_resp;

            str_action = ls_comps[2];

            if str_action == "config.xml":
                obj_resp = self.GetResponse();
                obj_resp.text = "<node><name>" + str_comp_name + "</name><label>" + str_comp_name + "</label></node>";
                return obj_resp;

            raise ValueError( 'Node (computer) action "' + str_action + "' is not supported" );

        if str_url.startswith( 'job' + posixpath.sep ):
            ls_jobs = str_url.split( posixpath.sep );
            str_job_name = ls_jobs[1];
            str_job_action = ls_jobs[2];

            if str_job_action == "config.xml" :
                if not str_job_name in self.dict_jobs_exist:
                    jerr = JenkinsError( str_job_name + " not found" );
                    jerr.code = 404;
                    raise jerr;

                obj_rsp = self.GetResponse();
                obj_rsp.text = self.dict_jobs_exist[ str_job_name ];

                if obj_rsp.text is None:
                    obj_rsp.text = "<job><name>" + str_job_name + "</name><disabled>" + ( "true" if ( str_job_name in self.ls_jobs_disabled ) else "false" ) + "</disabled><posBuild><action><disabled>true</disabled></action></postBuild></job>";

                return obj_rsp;

            raise ValueError( 'Job action "' + str_action + '" is not supported' );

        raise ValueError( "URL '" + str_url + "' is not supported." );
    
class QueueItemTestSuite(unittest.TestCase):
    
    def test_initial_state(self):
        mock_response="""
        {
            "task": {
                "name": "job"
            }
        }
        """
        item=QueueItem(1, TestClient([MockResponse(200, {}, mock_response),
                                      MockResponse(200, {}, mock_response),]))
        self.assertFalse(item.is_running())
        self.assertEqual(1, item.queue_id)
        with self.assertRaises(BuildStatusException):
            _=item.get_build()

    def test_invalid_queue_id(self):
        item=QueueItem(1, TestClient([MockResponse(404, {}, "{}")]))
        with self.assertRaises(JenkinsError):
            item.is_running()

    def test_build_still_queued(self):
        mock_response="""
        {
            "task": {
                "name": "job"
            }
        }
        """
        item=QueueItem(1, TestClient([MockResponse(200, {}, mock_response),
                                      MockResponse(200, {}, mock_response),]))
        self.assertFalse(item.is_running())
        with self.assertRaises(BuildStatusException):
            _=item.get_build()
            
    def test_build_started(self):
        mock_response="""
        {
            "task": {
                "name": "job"
            }, 
            "executable": {
                "number": 42
            }
        }
        """
        item=QueueItem(1, TestClient([MockResponse(200, {}, mock_response),
                                      MockResponse(200, {}, mock_response),]))
        self.assertTrue(item.is_running())
        build=item.get_build()
        self.assertEqual("job", build.job_name)
        self.assertEqual(42, build.build_number)

    def test_queue_status_updated(self):
        mock_response_1="""
        {
            "task": {
                "name": "job"
            }
        }
        """
        mock_response_2="""
        {
            "task": {
                "name": "job"
            }, 
            "executable": {
                "number": 42
            }
        }
        """
        item=QueueItem(1, TestClient([MockResponse(200, {}, mock_response_1),
                                      MockResponse(200, {}, mock_response_2),]))
        self.assertFalse(item.is_running())
        self.assertTrue(item.is_running())

    def test_jenkins_response_parsed(self):
        response=MockResponse(200, {"Location": "http://ci.com:123/queue/item/42/"}, [])
        item=QueueItem.from_jenkins_response(response, TestClient())
        self.assertEqual(42, item.queue_id)


class BuildTestSuite(unittest.TestCase):

    def test_running_build_status(self):
        mock_response="""
        {"result": null}
        """
        build=Build("job", 42, TestClient([MockResponse(200, {}, mock_response, )]))
        self.assertEqual("job", build.job_name)
        self.assertEqual(42, build.build_number)
        self.assertEqual(BuildStatus.BUILDING, build.get_status())

    def test_finished_build_status(self):
        mock_response="""
        {"result": "FAILURE"}
        """
        build=Build("job", 42, TestClient([MockResponse(200, {}, mock_response, )]))
        self.assertEqual("job", build.job_name)
        self.assertEqual(42, build.build_number)
        self.assertEqual(BuildStatus.FAILURE, build.get_status())

    def test_build_status_updated(self):
        mock_response_1="""
        {"result": null}
        """
        mock_response_2="""
        {"result": "SUCCESS"}
        """
        build=Build("job", 42, TestClient([MockResponse(200, {}, mock_response_1),
                                           MockResponse(200, {}, mock_response_2)]))
        self.assertEqual(BuildStatus.BUILDING, build.get_status())
        self.assertEqual(BuildStatus.SUCCESS, build.get_status())

    def test_invalid_build_params(self):
        build=Build("job", 42, TestClient([MockResponse(404, {}, "{}")]))
        with self.assertRaises(JenkinsError):
            build.get_status()

def xml_norm( str_xml ):
    parm_type = str (type (str_xml) )
    str_cmp = str_xml

    if parm_type != "<class 'str'>" and sys.version_info.major == 3:
        str_cmp = str (str_xml, 'UTF-8')

    str_cmp = re.sub( r'<\?xml[^<>]*>', '', str_cmp );
    str_cmp = re.sub( r'[\n\r]*', '', str_cmp );

    return str_cmp


class XMLTestSuite( unittest.TestCase ):
    def setUp( r_self ):
        os.environ[ 'JENKINS_URL' ] = 'http://localhost:8080';

    def test_get_node_label_from_config( self ):
        jnk = Jenkins();
        self.assertEqual( jnk.get_node_label_from_config( "<node><label>test</label></node>" ), "test" );
        self.assertEqual( jnk.get_node_label_from_config( "<node><label>test best babs</label></node>" ), "test best babs" );
        self.assertEqual( "", jnk.get_node_label_from_config( "<node><label></label></node>" ) );
        self.assertEqual( "", jnk.get_node_label_from_config( "<node><label/></node>" ) );
        self.assertEqual( "", jnk.get_node_label_from_config( "<node><blablabla/><bababa>bububu</bababa></node>" ) );
        self.assertEqual( "", jnk.get_node_label_from_config( "<node><buka><chacha/><label>baba</label></buka></node>" ) );
        self.assertEqual( jnk.get_node_label_from_config( "<node><buka><chacha/><label>baba</label></buka><label>ka</label></node>" ), "ka" );
        return;

    def test_set_node_label_in_config( self ):
        jnk = Jenkins();
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node><label>bukachacha</label></node>" , "chachabuka" ) ), "<node><label>chachabuka</label></node>" );
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node><label>bu ka cha cha</label></node>" , "chachabuka" ) ), "<node><label>chachabuka</label></node>" );
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node><label>bu ka</label></node>" , "chachabuka" ) ), "<node><label>chachabuka</label></node>" );
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node><label></label></node>" , "chachabuka" ) ), "<node><label>chachabuka</label></node>" );
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node><label/></node>" , "chachabuka" ) ), "<node><label>chachabuka</label></node>" );
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node></node>" , "chachabuka" ) ), "<node><label>chachabuka</label></node>" );
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node/>" , "chachabuka" ) ), "<node><label>chachabuka</label></node>" );
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node><label>bukachacha</label></node>" , "cha cha bu ka" ) ), "<node><label>cha cha bu ka</label></node>" );
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node><label>bu ka cha cha</label></node>" , "cha cha bu ka" ) ), "<node><label>cha cha bu ka</label></node>" );
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node><label>bu ka</label></node>" , "cha cha bu ka" ) ), "<node><label>cha cha bu ka</label></node>" );
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node><label></label></node>" , "cha cha bu ka" ) ), "<node><label>cha cha bu ka</label></node>" );
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node><label/></node>" , "cha cha bu ka" ) ), "<node><label>cha cha bu ka</label></node>" );
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node></node>" , "cha cha bu ka" ) ), "<node><label>cha cha bu ka</label></node>" );
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node/>" , "cha cha bu ka" ) ), "<node><label>cha cha bu ka</label></node>" );
        self.assertEqual( xml_norm( jnk.set_node_label_in_config( "<node><buka><label>ka</label></buka></node>" , "cha cha bu ka" ) ), "<node><buka><label>ka</label></buka><label>cha cha bu ka</label></node>" );
        return;

    def test_xml_job_enable( self ):
        jnk = Jenkins();
        self.assertEqual( xml_norm( jnk.job_xml_enable( "<job><name>krab</name><disabled>true</disabled><attr>ratcat</attr></job>" ) ), "<job><name>krab</name><disabled>false</disabled><attr>ratcat</attr></job>" );
        self.assertEqual( xml_norm( jnk.job_xml_enable( "<job><name>krab</name><attr>ratcat</attr></job>" ) ), "<job><name>krab</name><attr>ratcat</attr></job>" );
        self.assertEqual( xml_norm( jnk.job_xml_enable( "<job><name>krab</name><disabled>true</disabled><attr>ratcat</attr><byaka><disabled>true</disabled></byaka></job>" ) ), "<job><name>krab</name><disabled>false</disabled><attr>ratcat</attr><byaka><disabled>true</disabled></byaka></job>" );
        self.assertEqual( xml_norm( jnk.job_xml_enable( "<job/>" ) ), "<job/>" );
        return;

class NodeTestSuite( unittest.TestCase ):
    def setUp( r_self ):
        os.environ[ 'JENKINS_URL' ] = 'http://localhost:8080';

    def test_node_enable( self ):
        self.assertEqual( TestJenkinsClient().enable_node( 'testnode' ), ( posixpath.join( 'computer', 'testnode', 'enable' ), {} ) );
        return;

    def test_node_disable( self ):
        self.assertEqual( TestJenkinsClient().disable_node( 'testnode' ), ( posixpath.join( 'computer', 'testnode', 'disable' ), {} ) );
        return;


    def test_node_delete( self ):
        self.assertEqual( TestJenkinsClient().delete_node( 'testnode' ), ( posixpath.join( 'computer', 'testnode', 'doDelete' ), {} ) );
        return;

    def test_node_get_label( self ):
        jnk = TestJenkinsClient();
        self.assertEqual( jnk.get_node_label( 'testnode', '<node><label>test node off</label><name>testnode</name></node>' ), [ 'test', 'node', 'off' ] )
        self.assertEqual( jnk.get_node_label( 'testnode', '<node><label>test node off</label><name>testnode</name><pa><label>bobby must die</label></pa></node>' ), [ 'test', 'node', 'off' ] )
        self.assertEqual( [''], jnk.get_node_label( 'testnode', '<node><name>testnode</name><pa><label>bobby must die</label></pa></node>' ) )
        self.assertEqual( [''], jnk.get_node_label( 'testnode', '<node/>' ) );
        self.assertEqual( jnk.get_node_label( 'testnode' ), [ 'testnode' ] ); #configuration is to be taken from Jenkins
        return;

    def test_node_set_label( self ):
        jnk = TestJenkinsClient();
        rsp = jnk.set_node_label( 'testnode', ['bu', 'ga', 'ga'] );
        self.assertEqual( rsp[0], posixpath.join( 'computer', 'testnode', 'config.xml' ) );
        self.assertEqual( xml_norm( rsp[1][ 'data' ] ), "<node><name>testnode</name><label>bu ga ga</label></node>" );
        self.assertEqual( rsp[1][ 'params' ], None );
        self.assertEqual( rsp[1][ 'headers' ], { 'content-type' : 'application/xml' } );
        return;

class JobTestSuite( unittest.TestCase ):
    def setUp( r_self ):
        os.environ[ 'JENKINS_URL' ] = 'http://localhost:8080';

    def test_job_get( self ):
        jnk = TestJenkinsClient();
        jnk.ls_jobs_disabled.append( 'test_job' );
        jnk.dict_jobs_exist[ 'test_job' ] = None;
        self.assertEqual( xml_norm( jnk.get_job( 'test_job' ) ), "<job><name>test_job</name><disabled>true</disabled><posBuild><action><disabled>true</disabled></action></postBuild></job>" );
        jnk.dict_jobs_exist[ 'pessed_job' ] = None;
        self.assertEqual( xml_norm( jnk.get_job( 'pessed_job' ) ), "<job><name>pessed_job</name><disabled>false</disabled><posBuild><action><disabled>true</disabled></action></postBuild></job>" );
        return;

    def test_job_create( self ):
        jnk = TestJenkinsClient();

        #nonexistant
        rsp = jnk.put_job( 'nonexistant', "<job><name>nonexistant</name><disabled>false</disabled></job>" );
        self.assertEqual( rsp[0], "createItem" );
        self.assertEqual( rsp[1][ 'headers' ], {'content-type' : 'application/xml'} );
        self.assertEqual( xml_norm( rsp[1][ 'data' ] ), "<job><name>nonexistant</name><disabled>false</disabled></job>" );
        self.assertEqual( rsp[1][ 'params' ], { 'name' : 'nonexistant' } );

        #existang
        jnk.dict_jobs_exist[ 'existant' ] = None;
        rsp = jnk.put_job( 'existant', "<job><name>existant</name><disabled>false</disabled></job>" );
        self.assertEqual( rsp[0], posixpath.join( 'job', 'existant', 'config.xml' ) );
        self.assertEqual( rsp[1][ 'headers' ], {'content-type' : 'application/xml'} );
        self.assertEqual( xml_norm( rsp[1][ 'data' ] ), "<job><name>existant</name><disabled>false</disabled></job>" );
        self.assertEqual( rsp[1][ 'params' ], None );

        return;

    def test_job_delete( self ):
        self.assertEqual( TestJenkinsClient().delete_job( 'testjob' )[ 0 ], posixpath.join( 'job', 'testjob', 'doDelete' ) );
        return;

    def test_job_copy( self ):
        jnk = TestJenkinsClient();
        dict_rsp_headers = { 'content-type' : 'application/xml' };

        # 1. template = no, existant config, new
        str_job_name = "s.job.case.01.test";
        str_job_xml = "<job><name>" + str_job_name + "</name><disabled>true</disabled><command>echo lazhaa</command><report><name>report</name><disabled>true</disabled></report></job>";
        obj_resp = jnk.copy_job( str_job_xml, str_job_name, enabled = False );
        self.assertEqual( obj_resp[0], "createItem" );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertEqual( obj_resp[1][ 'params' ], { 'name' : str_job_name } );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_xml );

        # 2. template = no, existant config, overwrite
        str_job_name = "s.job.case.02.test";
        jnk.dict_jobs_exist[ str_job_name ] = None;
        str_job_xml = "<job><name>" + str_job_name + "</name><disabled>true</disabled><command>echo lazhaa</command><report><name>report</name><disabled>true</disabled></report></job>";
        obj_resp = jnk.copy_job( str_job_xml, str_job_name, enabled = False );
        self.assertEqual( obj_resp[0], posixpath.join( "job", str_job_name, "config.xml" ) );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertIsNone( obj_resp[1][ 'params' ] );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_xml );

        # 3. template = no, get config, new
        str_job_dst_name = "s.job.case.03.1.test";
        str_job_src_name = "s.job.case.03.2.test";
        jnk.dict_jobs_exist[ str_job_src_name ] = "<job><name>" + str_job_src_name + "</name><disabled>true</disabled><command>echo biliberda</command></job>";
        str_job_dst_xml = "<job><name>" + str_job_src_name + "</name><disabled>true</disabled><command>echo biliberda</command></job>";
        obj_resp = jnk.copy_job( str_job_src_name, str_job_dst_name, enabled = False );
        self.assertEqual( obj_resp[0], "createItem" );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertEqual( obj_resp[1][ 'params' ], { "name" : str_job_dst_name } );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_dst_xml );

        # 4. template = no, get config, overwrite
        str_job_dst_name = "s.job.case.03.1.test";
        str_job_src_name = "s.job.case.03.2.test";
        jnk.dict_jobs_exist[ str_job_dst_name ] = None;
        jnk.dict_jobs_exist[ str_job_src_name ] = "<job><name>" + str_job_src_name + "</name><disabled>true</disabled><command>echo biliberda</command></job>";
        str_job_dst_xml = "<job><name>" + str_job_src_name + "</name><disabled>true</disabled><command>echo biliberda</command></job>";
        obj_resp = jnk.copy_job( str_job_src_name, str_job_dst_name, enabled = False );
        self.assertEqual( obj_resp[0], posixpath.join( "job", str_job_dst_name, "config.xml" ) );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertIsNone( obj_resp[1][ 'params' ] );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_dst_xml );

        # 5. template = yes, existant config, new
        str_job_name = "s.job.case.05.test";
        str_job_xml_src = "<job><name>" + str_job_name + "</name><disabled>true</disabled><command>echo ${blablabla} ${tups}</command><report><name>report</name><disabled>true</disabled></report></job>";
        str_job_xml_dst = "<job><name>" + str_job_name + "</name><disabled>true</disabled><command>echo babs stupid shark</command><report><name>report</name><disabled>true</disabled></report></job>";
        obj_resp = jnk.copy_job( str_job_xml_src, str_job_name, parameters = { "blablabla" : "babs", "tups" : "stupid shark" }, use_template = True, enabled = False );
        self.assertEqual( obj_resp[0], "createItem" );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertEqual( obj_resp[1][ 'params' ], { 'name' : str_job_name } );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_xml_dst );

        # 6. template = yes, existant config, overwrite
        str_job_name = "s.job.case.06.test";
        jnk.dict_jobs_exist[ str_job_name ] = None;
        str_job_xml_src = "<job><name>" + str_job_name + "</name><disabled>true</disabled><command>echo ${blablabla} ${tups}</command><report><name>report</name><disabled>true</disabled></report></job>";
        str_job_xml_dst = "<job><name>" + str_job_name + "</name><disabled>true</disabled><command>echo babs stupid shark</command><report><name>report</name><disabled>true</disabled></report></job>";
        obj_resp = jnk.copy_job( str_job_xml_src, str_job_name, parameters = { "blablabla" : "babs", "tups" : "stupid shark" }, use_template = True, enabled = False );
        self.assertEqual( obj_resp[0], posixpath.join( "job", str_job_name, "config.xml" ) );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertIsNone( obj_resp[1][ 'params' ] );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_xml_dst );

        # 7. template = yes, get config, new
        str_job_src_name = "s.job.case.07.1.test";
        str_job_dst_name = "s.job.case.07.2.test";
        str_job_xml_src = "<job><name>" + str_job_src_name + "</name><disabled>true</disabled><command>echo ${blablabla} ${tups}</command><report><name>report</name><disabled>true</disabled></report></job>";
        jnk.dict_jobs_exist[ str_job_src_name ] = str_job_xml_src;
        str_job_xml_dst = "<job><name>" + str_job_src_name + "</name><disabled>true</disabled><command>echo babarobot stupid shark</command><report><name>report</name><disabled>true</disabled></report></job>";
        obj_resp = jnk.copy_job( str_job_src_name, str_job_dst_name, parameters = { "blablabla" : "babarobot", "tups" : "stupid shark" }, use_template = True, enabled = False );
        self.assertEqual( obj_resp[0], "createItem" );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertEqual( obj_resp[1][ 'params' ], { 'name' : str_job_dst_name } );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_xml_dst );

        # 8. template = yes, get config, overwrite
        str_job_src_name = "s.job.case.08.1.test";
        str_job_dst_name = "s.job.case.08.2.test";
        jnk.dict_jobs_exist[ str_job_dst_name ] = None;
        str_job_xml_src = "<job><name>" + str_job_src_name + "</name><disabled>true</disabled><command>echo ${blablabla} ${tups}</command><report><name>report</name><disabled>false</disabled></report></job>";
        jnk.dict_jobs_exist[ str_job_src_name ] = str_job_xml_src;
        str_job_xml_dst = "<job><name>" + str_job_src_name + "</name><disabled>true</disabled><command>echo babarobot stupid shark</command><report><name>report</name><disabled>false</disabled></report></job>";
        obj_resp = jnk.copy_job( str_job_src_name, str_job_dst_name, parameters = { "blablabla" : "babarobot", "tups" : "stupid shark" }, use_template = True, enabled = False );
        self.assertEqual( obj_resp[0], posixpath.join( "job", str_job_dst_name, "config.xml" ) );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertIsNone( obj_resp[1][ 'params' ] );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_xml_dst );

        # 9. template = no, existant config, new, enable disabled
        str_job_name = "s.job.case.09.test";
        str_job_xml_src = "<job><name>" + str_job_name + "</name><disabled>true</disabled><command>echo lazhaa</command><report><name>report</name><disabled>true</disabled></report></job>";
        str_job_xml_dst = "<job><name>" + str_job_name + "</name><disabled>false</disabled><command>echo lazhaa</command><report><name>report</name><disabled>true</disabled></report></job>";
        obj_resp = jnk.copy_job( str_job_xml_src, str_job_name );
        self.assertEqual( obj_resp[0], "createItem" );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertEqual( obj_resp[1][ 'params' ], { 'name' : str_job_name } );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_xml_dst );

        # 10. template = no, existant config, overwrite, enable disabled
        str_job_name = "s.job.case.10.test";
        jnk.dict_jobs_exist[ str_job_name ] = None;
        str_job_xml_src = "<job><name>" + str_job_name + "</name><disabled>true</disabled><command>echo lazhaa</command><report><name>report</name><disabled>true</disabled></report></job>";
        str_job_xml_dst = "<job><name>" + str_job_name + "</name><disabled>false</disabled><command>echo lazhaa</command><report><name>report</name><disabled>true</disabled></report></job>";
        obj_resp = jnk.copy_job( str_job_xml_src, str_job_name );
        self.assertEqual( obj_resp[0], posixpath.join( "job", str_job_name, "config.xml" ) );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertIsNone( obj_resp[1][ 'params' ] );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_xml_dst );

        # 11. template = no, get config, new, enable disabled
        str_job_dst_name = "s.job.case.11.1.test";
        str_job_src_name = "s.job.case.11.2.test";
        jnk.dict_jobs_exist[ str_job_src_name ] = "<job><name>" + str_job_src_name + "</name><disabled>true</disabled><command>echo biliberda</command></job>";
        str_job_dst_xml = "<job><name>" + str_job_src_name + "</name><disabled>false</disabled><command>echo biliberda</command></job>";
        obj_resp = jnk.copy_job( str_job_src_name, str_job_dst_name );
        self.assertEqual( obj_resp[0], "createItem" );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertEqual( obj_resp[1][ 'params' ], { "name" : str_job_dst_name } );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_dst_xml );

        # 12. template = no, get config, overwrite, enable disabled
        str_job_dst_name = "s.job.case.12.1.test";
        str_job_src_name = "s.job.case.12.2.test";
        jnk.dict_jobs_exist[ str_job_dst_name ] = None;
        jnk.dict_jobs_exist[ str_job_src_name ] = "<job><name>" + str_job_src_name + "</name><disabled>true</disabled><command>echo biliberda</command><report><name>report</name><disabled>true</disabled></report></job>";
        str_job_dst_xml = "<job><name>" + str_job_src_name + "</name><disabled>false</disabled><command>echo biliberda</command><report><name>report</name><disabled>true</disabled></report></job>";
        obj_resp = jnk.copy_job( str_job_src_name, str_job_dst_name );
        self.assertEqual( obj_resp[0], posixpath.join( "job", str_job_dst_name, "config.xml" ) );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertIsNone( obj_resp[1][ 'params' ] );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_dst_xml );

        # 13. template = yes, existant config, new, enable disabled
        str_job_name = "s.job.case.13.test";
        str_job_xml_src = "<job><name>" + str_job_name + "</name><disabled>true</disabled><command>echo ${taps} ${tups}</command><report><name>report</name><disabled>true</disabled></report></job>";
        str_job_xml_dst = "<job><name>" + str_job_name + "</name><disabled>false</disabled><command>echo cant pencil stupid shark</command><report><name>report</name><disabled>true</disabled></report></job>";
        obj_resp = jnk.copy_job( str_job_xml_src, str_job_name, parameters = { "taps" : "cant pencil", "tups" : "stupid shark" }, use_template = True );
        self.assertEqual( obj_resp[0], "createItem" );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertEqual( obj_resp[1][ 'params' ], { 'name' : str_job_name } );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_xml_dst );

        # 14. template = yes, existant config, overwrite, enable disabled
        str_job_name = "s.job.case.14.test";
        jnk.dict_jobs_exist[ str_job_name ] = None;
        str_job_xml_src = "<job><name>" + str_job_name + "</name><disabled>true</disabled><command>echo ${blablabla} ${tups}</command><report><name>report</name><disabled>true</disabled></report></job>";
        str_job_xml_dst = "<job><name>" + str_job_name + "</name><disabled>false</disabled><command>echo babs stupid shark</command><report><name>report</name><disabled>true</disabled></report></job>";
        obj_resp = jnk.copy_job( str_job_xml_src, str_job_name, parameters = { "blablabla" : "babs", "tups" : "stupid shark" }, use_template = True );
        self.assertEqual( obj_resp[0], posixpath.join( "job", str_job_name, "config.xml" ) );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertIsNone( obj_resp[1][ 'params' ] );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_xml_dst );

        # 15. template = yes, get config, new
        str_job_src_name = "s.job.case.15.1.test";
        str_job_dst_name = "s.job.case.15.2.test";
        str_job_xml_src = "<job><name>" + str_job_src_name + "</name><disabled>true</disabled><command>python2.7 ${faak}.py ${tups}.py</command><report><name>report</name><disabled>true</disabled></report></job>";
        jnk.dict_jobs_exist[ str_job_src_name ] = str_job_xml_src;
        str_job_xml_dst = "<job><name>" + str_job_src_name + "</name><disabled>false</disabled><command>python2.7 pussy pig.py stupid shark.py</command><report><name>report</name><disabled>true</disabled></report></job>";
        obj_resp = jnk.copy_job( str_job_src_name, str_job_dst_name, parameters = { "faak" : "pussy pig", "tups" : "stupid shark" }, use_template = True );
        self.assertEqual( obj_resp[0], "createItem" );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertEqual( obj_resp[1][ 'params' ], { 'name' : str_job_dst_name } );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_xml_dst );

        # 16. template = yes, get config, overwrite
        str_job_src_name = "s.job.case.16.1.test";
        str_job_dst_name = "s.job.case.16.2.test";
        jnk.dict_jobs_exist[ str_job_dst_name ] = None;
        str_job_xml_src = "<job><name>" + str_job_src_name + "</name><disabled>true</disabled><command>python2.7 ${kill_them_all}.py ${dirty_bugs}</command><report><name>report</name><disabled>false</disabled></report></job>";
        jnk.dict_jobs_exist[ str_job_src_name ] = str_job_xml_src;
        str_job_xml_dst = "<job><name>" + str_job_src_name + "</name><disabled>false</disabled><command>python2.7 be_nice_gay.py --destroy-brain=True</command><report><name>report</name><disabled>false</disabled></report></job>";
        obj_resp = jnk.copy_job( str_job_src_name, str_job_dst_name, parameters = { "kill_them_all" : "be_nice_gay", "dirty_bugs" : "--destroy-brain=True" }, use_template = True );
        self.assertEqual( obj_resp[0], posixpath.join( "job", str_job_dst_name, "config.xml" ) );
        self.assertEqual( obj_resp[1][ 'headers' ], dict_rsp_headers );
        self.assertIsNone( obj_resp[1][ 'params' ] );
        self.assertEqual( xml_norm( obj_resp[1][ 'data' ] ), str_job_xml_dst );
        return;

    def test_job_enable( self ):
        self.assertEqual( TestJenkinsClient().enable_job( 'testjob' )[ 0 ], posixpath.join( 'job', 'testjob', 'enable' ) );
        return;

    def test_job_disable( self ):
        self.assertEqual( TestJenkinsClient().disable_job( 'testjob' )[ 0 ], posixpath.join( 'job', 'testjob', 'disable' ) );
        return;

    def test_list_builds( self ):
        ## test for single node
        result = TestJenkinsClient().list_builds( 'test-node' );
        self.assertIn( "test-node", result );
        self.assertEqual( len( result[ "test-node" ] ), 1 );
        self.assertEqual( result[ "test-node" ][0][ "job_name" ], "c.TEST_CUSTOMER.branches-test.test" );
        self.assertEqual( result[ "test-node" ][0][ "build_nubmer" ], 27 );
        ## test for system-wide
        result = TestJenkinsClient().list_builds();
        self.assertIn( "test-node", result );
        self.assertEqual( len( result[ "test-node" ] ), 1 );
        self.assertEqual( result[ "test-node" ][0][ "job_name" ], "c.TEST_CUSTOMER.branches-test.test" );
        self.assertEqual( result[ "test-node" ][0][ "build_nubmer" ], 27 );
        self.assertIn( "other-node", result );
        self.assertEqual( len( result[ "other-node" ] ), 1 );
        self.assertEqual( result[ "other-node" ][0][ "job_name" ], "c.TEST_CUSTOMER.trunk.test" );
        self.assertEqual( result[ "other-node" ][0][ "build_nubmer" ], 3 );
        return;

    def test_list_queue( self ):
        result = TestJenkinsClient().list_queue();
        self.assertEqual( len( result ), 2 );
        jobnames = map( lambda x: x.job_name, result );
        self.assertIn( "c.OWI_LUXCTRL.branches-int.cards", jobnames );
        self.assertIn( "tech.selfcheck", jobnames );
        jobids = map( lambda x: x.queue_id, result );
        self.assertIn( 9590, jobids);
        self.assertIn( 9559, jobids);

    def test_unqueue( self ):
        jnk = TestJenkinsClient();
        jnk.unqueue_item( 1234 );

        self.assertEqual( jnk.last_post_url, posixpath.join( 'queue', 'cancelItem' ) );
        self.assertEqual( jnk.last_post_data[ "params" ][ "id" ], 1234 );
        self.assertEqual( jnk.last_post_data[ "data" ], dict() );

    def test_list_node( self ):
        jnk = TestJenkinsClient();
        result = jnk.list_nodes();

        for str_node in jnk.computer_data.keys():
            self.assertIn( str_node, result );





from oc_cdtapi import NexusAPI
import posixpath
from copy import deepcopy
from xml.etree import ElementTree
import re
import json

class ResponseMock(object):
    def __init__(self, url, status_code, content = None):
        self.status_code = status_code
        self.content = content
        self.url = url

    def json(self):

        if not self.content:
            return None

        return json.loads(self.content)

class NexusAPILsMock(object):
    """
    Helper class for Nexus API responses on test LS capabilities
    """
    def __init__(self, url=None):
        # list of artifacts to provide
        # four groups are enough
        self._url = url
        self.packaging = ["jar", "zip", "tgz", "rpm"]
        self.classifiers = ["cl", "vx"]
        self.groups = len(self.packaging)
        self.artifacts = len(self.packaging)
        self.versions = len(self.packaging)
        self.__artifacts = list()

        # generating some test artifacts
        for __group in list(map(lambda x: "test.group.%d.id" % x, range(0, self.groups))):
            for __artifact in list(
                    map(lambda x: ":".join([__group, "artifact-%d" % x]), range(0, self.artifacts))):
                for __version in list(
                        map(lambda x: ":".join([__artifact, "%d.%d.%d" % (x, x+x, x*x)]), range(0, self.versions))):
                    for __packaging in list(
                            map(lambda x: ":".join([__version, self.packaging[x]]), range(0, len(self.packaging)))):
                        for __classifier in range(0, len(self.classifiers) + 1):
                            if __classifier >= len(self.classifiers):
                                self.__artifacts.append(NexusAPI.parse_gav(__packaging))
                                continue

                            self.__artifacts.append(NexusAPI.parse_gav(
                                ":".join([__packaging, self.classifiers[__classifier]])))

    @property
    def __is_nexus(self):
        return self._url.rstrip(posixpath.sep).endswith(posixpath.sep + "nexus")

    def _strip_request_url(self, url):
        if not url.startswith(self._url):
            raise ValueError("Incorrect request URL")

        # NOTE: this should be done by urllib.urlparse,
        #       but done so ugly to get rid of extra dependencies
        url = url[len(self._url):].strip(posixpath.sep)
        return url

    def _filter_artifacts(self, params):

        _res = deepcopy(self.__artifacts)

        for __p in ['g', 'a', 'v', 'p', 'c']:
            if not params.get(__p):
                continue

            _res = list(filter(lambda x: x.get(__p) == params.get(__p), _res))

        return _res

    def __xml_search_response(self, lst, too_many_results):
        _rsp = ElementTree.Element("searchNGResponse")
        _tc = ElementTree.SubElement(_rsp, "totalCount")
        _tc.text = str(len(lst))
        _tmr = ElementTree.SubElement(_rsp, "tooManyResults")
        _tmr.text = str(too_many_results).lower()
        _data = ElementTree.SubElement(_rsp, "data")

        if not len(lst):
            return _rsp

        _gav = list()

        for _af in lst:
            _dct = dict()

            for _k in ["g", "a", "v"]:
                _dct[_k] = _af.get(_k)

            _gav_s = NexusAPI.gav_to_str(_dct)

            if _gav_s in _gav:
                continue

            _gav.append(_gav_s)

            _afx = ElementTree.SubElement(_data, "artifact")
            ElementTree.SubElement(_afx, "groupId").text = _af.get("g")
            ElementTree.SubElement(_afx, "artifactId").text = _af.get("a")
            ElementTree.SubElement(_afx, "version").text = _af.get("v")

            # add "pom" packaging
            _af_hits = ElementTree.SubElement(_afx, "artifactHits")
            _af_hit = ElementTree.SubElement(_af_hits, "artifactHit")


            _af_links = ElementTree.SubElement(_af_hit, "artifactLinks")

            for _pkg in ["pom", _af.get("p")]:
                _af_link = ElementTree.SubElement(_af_links, "artifactLink")

                ElementTree.SubElement(_af_link, "extension").text = _pkg

                if _pkg == "pom" or not _af.get("c"):
                    continue

                ElementTree.SubElement(_af_link, "classifier").text = _af.get("c")

            if too_many_results: 
                continue

            for _afs in list(filter(lambda x: 
                all(list(map(lambda y: x.get(y)==_af.get(y), ["g", "a", "v"]))), lst)):

                if all(list(map(lambda x: _afs.get(x) == _af.get(x), ["p", "c"]))):
                    continue

                _af_link = ElementTree.SubElement(_af_links, "artifactLink")
                ElementTree.SubElement(_af_link, "extension").text = _afs.get("p")

                if not _afs.get("c"):
                    continue

                ElementTree.SubElement(_af_link, "classifier").text = _afs.get("c")

        return _rsp

    def get(self, req, params=None, data=None, files=None, headers=None, **kvargs):
        __req = self._strip_request_url(req)

        if __req != posixpath.join("service", "local", "lucene", "search"):
            raise ValueError("Unsupported request: %s" % req)

        if not params:
            raise ValueError("Search parameters shoud not be empty")

        _pres = self._filter_artifacts(params)

        return ResponseMock(req, 200, 
                content=ElementTree.tostring(
                    self.__xml_search_response(_pres, too_many_results=("a" not in params.keys()))))

    def head(self, req, params=None, data=None, files=None, headers=None, **kvargs):
        __req = self._strip_request_url(req)
        _apath = __req

        if self.__is_nexus:
            if not __req.startswith(posixpath.join('content', 'repositories')):
                raise ValueError("Unsupported request: %s" % req)

            _apath = posixpath.sep.join(__req.split(posixpath.sep)[len(["content", "repositories", "public"]):])

        __gav = NexusAPI.gav_from_path(_apath, dictionary=True)

        return ResponseMock(req, 200 if __gav in self.__artifacts else 404)


class ArtifactoryAPILsMock(NexusAPILsMock):
    def __json_search_response(self, lst):
        _res = list()

        for _a in lst:
            _pth = NexusAPI.gav_to_path(_a)
            _res.append({
                "uri": posixpath.join(self._url, "api", "storage", "public", _pth),
                "path": posixpath.sep + _pth })

        return {"results":_res}

    def get(self, req, params=None, data=None, files=None, headers=None, **kvargs):
        __req = self._strip_request_url(req)

        if __req != posixpath.join("api", "search", "gavc"):
            raise ValueError("Unsupported request: %s" % req)

        if not params:
            raise ValueError("Search parameters shoud not be empty")

        if 'p' in params:
            raise ValueError("Packaging search is not supported by Artifactory")

        if not headers or "X-Result-Detail" not in headers:
            raise ValueError("Suppose header 'X-Result-Detail' for Artifactory search")

        _pres = self._filter_artifacts(params)

        return ResponseMock(req, 200, content=json.dumps(self.__json_search_response(_pres)))



from unittest import TestCase

from oc_cdtapi import NexusAPI

from xml.etree import ElementTree

class GavToPathTestSuite(TestCase):
    def test_full_gav_converted_from_unicode (self):
        path = NexusAPI.gav_to_path(u'group.id:artifact:0.1:pkg:classifier')
        self.assertEqual('group/id/artifact/0.1/artifact-0.1-classifier.pkg', path)

    def test_full_gav_converted(self):
        path = NexusAPI.gav_to_path('group.id:artifact:0.1:pkg:classifier')
        self.assertEqual('group/id/artifact/0.1/artifact-0.1-classifier.pkg', path)

    def test_dict_gav_converted(self):
        gav = {"g": "group.id", "a": "artifact", "v": "0.1", "p": "pkg", "c": "classifier"}
        path = NexusAPI.gav_to_path(gav)
        self.assertEqual('group/id/artifact/0.1/artifact-0.1-classifier.pkg', path)

    def test_full_relaxed_conversion(self):
        path = NexusAPI.gav_to_path('group.id:artifact:0.1:pkg:classifier', relaxed=True)
        self.assertEqual('group/id/artifact/0.1/artifact-0.1-classifier.pkg', path)

    def test_converted_without_classifier(self):
        path = NexusAPI.gav_to_path('group.id:artifact:0.1:pkg')
        self.assertEqual('group/id/artifact/0.1/artifact-0.1.pkg', path)

    def test_default_packaging(self):
        path = NexusAPI.gav_to_path('group.id:artifact:0.1')
        self.assertEqual('group/id/artifact/0.1/artifact-0.1.jar', path)

    def test_no_version_relaxed(self):
        path = NexusAPI.gav_to_path('group.id:artifact', relaxed=True)
        self.assertEqual('group/id/artifact', path)

    def test_only_version_relaxed(self):
        path = NexusAPI.gav_to_path('group.id', relaxed=True)
        self.assertEqual('group/id', path)

    def test_only_artifact_relaxed(self):
        path = NexusAPI.gav_to_path({'a': 'art'}, relaxed=True)
        self.assertEqual('art', path)

    def test_gav_required_by_default(self):
        with self.assertRaises(ValueError):
            NexusAPI.gav_to_path({'g': 'group', 'a': 'art'})


class GavToFilenameTestSuite(TestCase):
    def test_filename_unicode (self):
        filename = NexusAPI.gav_to_filename(u'group.id:artifact:0.1:pkg:classifier')
        self.assertEqual("artifact-0.1-classifier.pkg", filename)

    def test_filename(self):
        filename = NexusAPI.gav_to_filename('group.id:artifact:0.1:pkg:classifier')
        self.assertEqual("artifact-0.1-classifier.pkg", filename)

    def test_classifier_is_optional(self):
        filename = NexusAPI.gav_to_filename('group.id:artifact:0.1:pkg')
        self.assertEqual("artifact-0.1.pkg", filename)

    def test_jar_is_default_packaging(self):
        filename = NexusAPI.gav_to_filename('group.id:artifact:0.1')
        self.assertEqual("artifact-0.1.jar", filename)

    def test_gav_required_for_filename(self):
        with self.assertRaises(ValueError):
            NexusAPI.gav_to_filename('group.id:artifact')


class GavToStrTestSuite(TestCase):

    def test_minimal_gav_written(self):
        gav_str = NexusAPI.gav_to_str({"g": "g", "a": "a", "v": "v"})
        self.assertEqual("g:a:v", gav_str)

    def test_packaged_gav_written(self):
        gav_str = NexusAPI.gav_to_str({"g": "g", "a": "a", "v": "v", "p": "p"})
        self.assertEqual("g:a:v:p", gav_str)

    def test_full_gav_written(self):
        gav_str = NexusAPI.gav_to_str({"g": "g", "a": "a", "v": "v", "p": "p", "c": "c"})
        self.assertEqual("g:a:v:p:c", gav_str)

    def test_classifier_needs_packaging(self):
        gav_str = NexusAPI.gav_to_str({"g": "g", "a": "a", "v": "v", "c": "c"})
        self.assertEqual("g:a:v", gav_str)

    def test_minimal_gav_required(self):
        with self.assertRaises(ValueError):
            NexusAPI.gav_to_str({"g": "g", "a": "a"})

    def test_classifier_none(self):
        gav_str = NexusAPI.gav_to_str({"g": "g", "a": "a", "v": "v", "p": "x", "c": ''})
        self.assertEqual(gav_str, "g:a:v:x")

class ParseGavTestSuite(TestCase):

    def test_full_gav_written(self):
        gav = NexusAPI.parse_gav('group.id:artifact.id:0.1.2.3:pkg')
        self.assertEqual(gav['a'], 'artifact.id')
        self.assertEqual(gav['g'], 'group.id')
        self.assertEqual(gav['p'], 'pkg')
        self.assertEqual(gav['v'], '0.1.2.3')
        self.assertIsNone(gav.get('c'))

    def test_empty_gav_written(self):
        with self.assertRaises(ValueError):
            gav = NexusAPI.parse_gav('')

    def test_parse_gav_no_pkg(self):
        gav = NexusAPI.parse_gav('group.id:artifact.id:0.1.2.3')
        self.assertEqual(gav['a'], 'artifact.id')
        self.assertEqual(gav['g'], 'group.id')
        self.assertEqual(gav['v'], '0.1.2.3')
        self.assertIsNone(gav.get('p'))
        self.assertIsNone(gav.get('c'))

    def test_parse_gav_no_v(self):
        with self.assertRaises(ValueError):
            gav = NexusAPI.parse_gav('group.id:artifact.id')

    def test_parse_gav_classifier(self):
        gav = NexusAPI.parse_gav('group.id:artifact.id:0.1.2.3:pkg:classifier')
        self.assertEqual(gav['a'], 'artifact.id')
        self.assertEqual(gav['c'], 'classifier')
        self.assertEqual(gav['g'], 'group.id')
        self.assertEqual(gav['p'], 'pkg')
        self.assertEqual(gav['v'], '0.1.2.3')

    def test_parse_gav_extra_member(self):
        with self.assertRaises(ValueError):
            gav = NexusAPI.parse_gav('group.id:artifact.id:0.1.2.3:pkg:classifier:wtf')

    def test_parse_gav_extra_member_relaxed(self):
        with self.assertRaises(ValueError):
            gav = NexusAPI.parse_gav('group.id:artifact.id:0.1.2.3:pkg:classifier:wtf', relaxed = True)

    def test_parse_gav_classifier_relaxed(self):
        gav = NexusAPI.parse_gav('group.id:artifact.id:0.1.2.3:pkg:classifier',relaxed = True)
        self.assertEqual(gav['a'], 'artifact.id')
        self.assertEqual(gav['c'], 'classifier')
        self.assertEqual(gav['g'], 'group.id')
        self.assertEqual(gav['p'], 'pkg')
        self.assertEqual(gav['v'], '0.1.2.3')
    
    def test_parse_gav_no_v_relaxed(self):
        gav = NexusAPI.parse_gav('group.id:artifact.id',relaxed = True)
        self.assertEqual(gav['g'], 'group.id')
        self.assertEqual(gav['a'], 'artifact.id')
        self.assertIsNone(gav.get('v'))
        self.assertIsNone(gav.get('p'))
        self.assertIsNone(gav.get('c'))
    
    def test_parse_gav_g_only_relaxed(self):
        gav = NexusAPI.parse_gav('group.id', relaxed = True)
        self.assertEqual(gav['g'], 'group.id')
        for _k in ['a', 'v', 'p', 'c']:
            self.assertNotIn(_k, list(gav.keys()))

    def test_parse_empty_gav_relaxed(self):
        self.assertEqual(NexusAPI.parse_gav('', relaxed = True), {})

    def test_parse_gav_dict_full(self):
        _dict = { 'g': 'groupId.test',
                  'a': 'artifactId-test',
                  'v': '0.0.0',
                  'p': 'pkg',
                  'c': 'classifierTest' }
        self.assertEqual(NexusAPI.parse_gav(_dict), _dict)

    def test_parse_gav_dict_short(self):
        _dict = { 'g': 'groupId.test',
                  'a': 'artifactId-test',
                  'v': '0.0.0'}
        self.assertEqual(NexusAPI.parse_gav(_dict), _dict)

    def test_parse_gav_dict_very_short(self):
        _dict = { 'g': 'groupId.test',
                  'v': '0.0.0'}
        with self.assertRaises(ValueError):
            NexusAPI.parse_gav(_dict)

    def test_parse_gav_dict_very_short(self):
        _dict = { 'g': 'groupId.test',
                  'v': '0.0.0'}
        self.assertEqual(NexusAPI.parse_gav(_dict, relaxed=True), _dict)


class GavToFileNameTestSuite(TestCase):
    def test_gav2fn_str_all(self):
        self.assertEqual('artifactId-0.1.2.3.4-classifier.pkg', 
                NexusAPI.gav_to_filename('groupId:artifactId:0.1.2.3.4:pkg:classifier'))
        self.assertEqual('artifactId-0.1.2.3.4-classifier.pkg', 
                NexusAPI.gav_to_filename('fake.group.id:artifactId:0.1.2.3.4:pkg:classifier'))

    def test_gav2fn_dict_all(self):
        self.assertEqual('artifactId-0.1.2.3.4-classifier.pkg', 
                NexusAPI.gav_to_filename({
                    'g': 'groupId', 
                    'a': 'artifactId',
                    'v': '0.1.2.3.4',
                    'p': 'pkg',
                    'c': 'classifier'}))

        self.assertEqual('artifactId-0.1.2.3.4-classifier.pkg',
                NexusAPI.gav_to_filename({
                    'g': 'fake.group.id', 
                    'a': 'artifactId',
                    'v': '0.1.2.3.4',
                    'p': 'pkg',
                    'c': 'classifier'}))

    def test_gav2fn_str_no_g(self):
        self.assertEqual('artifactId-0.1.2.3.4-classifier.pkg', 
                NexusAPI.gav_to_filename(':artifactId:0.1.2.3.4:pkg:classifier'))
        self.assertEqual('artifactId-0.1.2.3.4-classifier.pkg',
                NexusAPI.gav_to_filename(':artifactId:0.1.2.3.4:pkg:classifier'))

    def test_gav2fn_dict_no_g(self):
        self.assertEqual('artifactId-0.1.2.3.4-classifier.pkg', 
                NexusAPI.gav_to_filename({
                    'a': 'artifactId',
                    'v': '0.1.2.3.4',
                    'p': 'pkg',
                    'c': 'classifier'}))

        self.assertEqual('artifactId-0.1.2.3.4-classifier.pkg',
                NexusAPI.gav_to_filename({
                    'a': 'artifactId',
                    'v': '0.1.2.3.4',
                    'p': 'pkg',
                    'c': 'classifier'}))

    def test_gav2fn_str_no_c(self):
        self.assertEqual('artifactId-0.1.2.3.4.pkg', 
                NexusAPI.gav_to_filename('groupId:artifactId:0.1.2.3.4:pkg'))
        self.assertEqual('artifactId-0.1.2.3.4.pkg', 
                NexusAPI.gav_to_filename('fake.group.id:artifactId:0.1.2.3.4:pkg'))

    def test_gav2fn_dict_no_c(self):
        self.assertEqual('artifactId-0.1.2.3.4.pkg', 
                NexusAPI.gav_to_filename({
                    'g': 'groupId', 
                    'a': 'artifactId',
                    'v': '0.1.2.3.4',
                    'p': 'pkg'}))

        self.assertEqual('artifactId-0.1.2.3.4.pkg',
                NexusAPI.gav_to_filename({
                    'g': 'fake.group.id', 
                    'a': 'artifactId',
                    'v': '0.1.2.3.4',
                    'p': 'pkg'}))

    def test_gav2fn_str_no_v(self):
        with self.assertRaises(ValueError):
            NexusAPI.gav_to_filename('groupId:artifactId')
        with self.assertRaises(ValueError):
            NexusAPI.gav_to_filename('fake.group.id:artifactId')

    def test_gav2fn_dict_no_v(self):
        with self.assertRaises(ValueError):
            NexusAPI.gav_to_filename({
                    'g': 'groupId', 
                    'a': 'artifactId',
                    'p': 'pkg',
                    'c': 'classifier'})
        with self.assertRaises(ValueError):
            NexusAPI.gav_to_filename({
                    'g': 'fake.group.id', 
                    'a': 'artifactId',
                    'p': 'pkg',
                    'c': 'classifier'})

    def test_gav2fn_str_no_a(self):
        with self.assertRaises(ValueError):
            NexusAPI.gav_to_filename('groupId')
        with self.assertRaises(ValueError):
            NexusAPI.gav_to_filename('fake.group.id')

    def test_gav2fn_dict_no_a(self):
        with self.assertRaises(ValueError):
            NexusAPI.gav_to_filename({
                    'g': 'groupId', 
                    'v': '0.1.2.3.4',
                    'p': 'pkg',
                    'c': 'classifier'})

        with self.assertRaises(ValueError):
            NexusAPI.gav_to_filename({
                    'g': 'fake.group.id', 
                    'v': '0.1.2.3.4',
                    'p': 'pkg',
                    'c': 'classifier'})

    def test_gav2fn_str_no_p(self):
        self.assertEqual('artifactId-0.1.2.3.4.jar', 
                NexusAPI.gav_to_filename('groupId:artifactId:0.1.2.3.4'))
        self.assertEqual('artifactId-0.1.2.3.4.jar', 
                NexusAPI.gav_to_filename('fake.group.id:artifactId:0.1.2.3.4'))

    def test_gav2fn_dict_no_p(self):
        self.assertEqual('artifactId-0.1.2.3.4-classifier.jar', 
                NexusAPI.gav_to_filename({
                    'g': 'groupId', 
                    'a': 'artifactId',
                    'v': '0.1.2.3.4',
                    'c': 'classifier'}))

        self.assertEqual('artifactId-0.1.2.3.4-classifier.jar',
                NexusAPI.gav_to_filename({
                    'g': 'fake.group.id', 
                    'a': 'artifactId',
                    'v': '0.1.2.3.4',
                    'c': 'classifier'}))

class PomFromGavTestSuite(TestCase):
    def _parse_and_general_asserts(self, xmlstr, groupId, artifactId, version, packaging=None):
        xmlr = ElementTree.fromstring(xmlstr)
        # assert root tag
        # today we have 4.0.0 model version with a corresponding XML namespace
        _xmlns = '{http://maven.apache.org/POM/4.0.0}'
        self.assertEqual(_xmlns + 'project', xmlr.tag)
        self.assertEqual(xmlr.find(_xmlns + 'modelVersion').text, '4.0.0')

        # assert GAV - as dictionary
        self.assertEqual(xmlr.find(_xmlns + 'groupId').text, groupId)
        self.assertEqual(xmlr.find(_xmlns + 'artifactId').text, artifactId)
        self.assertEqual(xmlr.find(_xmlns + 'version').text, version)

        if packaging:
            self.assertEqual(xmlr.find(_xmlns + 'packaging').text, packaging)
        else:
            self.assertIsNone(xmlr.find(_xmlns + 'packaging'))

        return xmlr

    def test_pomFromGav_all_str(self):
        _pom = NexusAPI.pom_from_gav('groupId.1.template:artifactId-1:version-1:packaging-1:classifier-1', no_packaging=False)
        self._parse_and_general_asserts(_pom, 'groupId.1.template', 'artifactId-1', 'version-1', 'packaging-1')

    def test_pomFromGav_nopkg_str(self):
        _pom = NexusAPI.pom_from_gav('groupId.2.template:artifactId-2:version-2:packaging-2:classifier-2', no_packaging=True)
        self._parse_and_general_asserts(_pom, 'groupId.2.template', 'artifactId-2', 'version-2')

    def test_pomFromGav_short_str(self):
        _pom = NexusAPI.pom_from_gav('groupId.3.template:artifactId-3:version-3', no_packaging=False)
        self._parse_and_general_asserts(_pom, 'groupId.3.template', 'artifactId-3', 'version-3')

    def test_pomFromGav_all_dict(self):
        _test_num = 1
        _gav_d = {
                'g': 'test.group.id.%d' % _test_num,
                'a': 'artifact-id-%d' % _test_num,
                'v': '%d.%d.%d' % (_test_num, _test_num + _test_num, _test_num * _test_num),
                'p': 'pkg%d' % _test_num,
                'c': 'class-if-eyer-%d' % _test_num }

        _pom = NexusAPI.pom_from_gav(_gav_d, no_packaging=False)
        self._parse_and_general_asserts(_pom, _gav_d['g'], _gav_d['a'], _gav_d['v'], _gav_d['p'])

    def test_pomFromGav_nopkg_dict(self):
        _test_num = 2
        _gav_d = {
                'g': 'test.group.id.%d' % _test_num,
                'a': 'artifact-id-%d' % _test_num,
                'v': '%d.%d.%d' % (_test_num, _test_num + _test_num, _test_num * _test_num),
                'p': 'pkg%d' % _test_num,
                'c': 'class-if-eyer-%d' % _test_num }

        _pom = NexusAPI.pom_from_gav(_gav_d, no_packaging=True)
        self._parse_and_general_asserts(_pom, _gav_d['g'], _gav_d['a'], _gav_d['v'])

    def test_pomFromGav_short_dict(self):
        _test_num = 2
        _gav_d = {
                'g': 'test.group.id.%d' % _test_num,
                'a': 'artifact-id-%d' % _test_num,
                'v': '%d.%d.%d' % (_test_num, _test_num + _test_num, _test_num * _test_num)}

        _pom = NexusAPI.pom_from_gav(_gav_d, no_packaging=False)
        self._parse_and_general_asserts(_pom, _gav_d['g'], _gav_d['a'], _gav_d['v'])

class GavFromPathTestSuite(TestCase):
    def test_gav_string_generated_full(self):
        gav = NexusAPI.gav_from_path('/group/id/artifact/0.1/artifact-0.1-classifier.pkg', dictionary=False)
        self.assertEqual('group.id:artifact:0.1:pkg:classifier', gav)

    def test_gav_string_generated_no_classifier(self):
        gav = NexusAPI.gav_from_path('/group/id/artifact/0.1/artifact-0.1.pkg', dictionary=False)
        self.assertEqual('group.id:artifact:0.1:pkg', gav)

    def test_gav_dict_generated_full(self):
        gav = NexusAPI.gav_from_path('/group/id/artifact/0.1/artifact-0.1-classifier.pkg')
        self.assertDictEqual({'g': 'group.id', 'a': 'artifact', 'v': '0.1', 'p': 'pkg', 'c': 'classifier'}, gav)

    def test_gav_dict_generated_no_classifier(self):
        gav = NexusAPI.gav_from_path('/group/id/artifact/0.1/artifact-0.1.pkg')
        self.assertDictEqual({'g': 'group.id', 'a': 'artifact', 'v': '0.1', 'p': 'pkg'}, gav)

    def test_gav_dict_no_packaging(self):
        gav = NexusAPI.gav_from_path('group/test/help/id/factart/1.2.3.4.5.6/factart-1.2.3.4.5.6', dictionary=True)
        self.assertDictEqual(gav, {'g': 'group.test.help.id', 'a': 'factart', 'v': '1.2.3.4.5.6'})

    def test_gav_str_no_packaging(self):
        gav = NexusAPI.gav_from_path('group/test/help/id/factart/1.2.3.4.5.6/factart-1.2.3.4.5.6', dictionary=False)
        self.assertEqual(gav, 'group.test.help.id:factart:1.2.3.4.5.6')

